@@ -1,4 +1,4 @@
-/* Copyright (C) 2017-2018 Open Information Security Foundation
+/* Copyright (C) 2017-2020 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -20,16 +20,15 @@
 use std;
 use std::ffi::{CStr,CString};
 use nom;
-use nom::be_u32;
-use der_parser::der_read_element_header;
+use nom::IResult;
+use nom::number::streaming::be_u32;
+use der_parser::der::der_read_element_header;
+use der_parser::ber::BerClass;
 use kerberos_parser::krb5_parser;
 use kerberos_parser::krb5::{EncryptionType,ErrorCode,MessageType,PrincipalName,Realm};
-use crate::applayer;
+use crate::applayer::{self, *};
 use crate::core;
 use crate::core::{AppProto,Flow,ALPROTO_FAILED,ALPROTO_UNKNOWN,STREAM_TOCLIENT,STREAM_TOSERVER,sc_detect_engine_state_free};
-use crate::parser::*;
-
-use crate::log::*;
 
 #[repr(u32)]
 pub enum KRB5Event {
@@ -88,8 +87,7 @@ pub struct KRB5Transaction {
     /// The events associated with this transaction
     events: *mut core::AppLayerDecoderEvents,
 
-    logged: applayer::LoggerFlags,
-    detect_flags: applayer::TxDetectFlags,
+    tx_data: applayer::AppLayerTxData,
 }
 
 pub fn to_hex_string(bytes: &[u8]) -> String {
@@ -115,15 +113,15 @@ impl KRB5State {
 
     /// Parse a Kerberos request message
     ///
-    /// Returns The number of messages parsed, or -1 on error
+    /// Returns 0 in case of success, or -1 on error
     fn parse(&mut self, i: &[u8], _direction: u8) -> i32 {
         match der_read_element_header(i) {
             Ok((_rem,hdr)) => {
                 // Kerberos messages start with an APPLICATION header
-                if hdr.class != 0b01 { return 1; }
-                match hdr.tag {
+                if hdr.class != BerClass::Application { return 0; }
+                match hdr.tag.0 {
                     10 => {
-                        self.req_id = hdr.tag;
+                        self.req_id = 10;
                     },
                     11 => {
                         let res = krb5_parser::parse_as_rep(i);
@@ -142,7 +140,7 @@ impl KRB5State {
                         self.req_id = 0;
                     },
                     12 => {
-                        self.req_id = hdr.tag;
+                        self.req_id = 12;
                     },
                     13 => {
                         let res = krb5_parser::parse_tgs_rep(i);
@@ -161,7 +159,7 @@ impl KRB5State {
                         self.req_id = 0;
                     },
                     14 => {
-                        self.req_id = hdr.tag;
+                        self.req_id = 14;
                     },
                     15 => {
                         self.req_id = 0;
@@ -240,8 +238,7 @@ impl KRB5Transaction {
             id: id,
             de_state: None,
             events: std::ptr::null_mut(),
-            logged: applayer::LoggerFlags::new(),
-            detect_flags: applayer::TxDetectFlags::default(),
+            tx_data: applayer::AppLayerTxData::new(),
         }
     }
 }
@@ -276,7 +273,7 @@ pub fn test_weak_encryption(alg:EncryptionType) -> bool {
 
 /// Returns *mut KRB5State
 #[no_mangle]
-pub extern "C" fn rs_krb5_state_new() -> *mut std::os::raw::c_void {
+pub extern "C" fn rs_krb5_state_new(_orig_state: *mut std::os::raw::c_void, _orig_proto: AppProto) -> *mut std::os::raw::c_void {
     let state = KRB5State::new();
     let boxed = Box::new(state);
     return unsafe{std::mem::transmute(boxed)};
@@ -335,25 +332,6 @@ pub extern "C" fn rs_krb5_tx_get_alstate_progress(_tx: *mut std::os::raw::c_void
     1
 }
 
-#[no_mangle]
-pub extern "C" fn rs_krb5_tx_set_logged(_state: *mut std::os::raw::c_void,
-                                       tx: *mut std::os::raw::c_void,
-                                       logged: u32)
-{
-    let tx = cast_pointer!(tx,KRB5Transaction);
-    tx.logged.set(logged);
-}
-
-#[no_mangle]
-pub extern "C" fn rs_krb5_tx_get_logged(_state: *mut std::os::raw::c_void,
-                                       tx: *mut std::os::raw::c_void)
-                                       -> u32
-{
-    let tx = cast_pointer!(tx,KRB5Transaction);
-    return tx.logged.get();
-}
-
-
 #[no_mangle]
 pub extern "C" fn rs_krb5_state_set_tx_detect_state(
     tx: *mut std::os::raw::c_void,
@@ -443,9 +421,9 @@ pub extern "C" fn rs_krb5_probing_parser(_flow: *const Flow,
     match der_read_element_header(slice) {
         Ok((rem, ref hdr)) => {
             // Kerberos messages start with an APPLICATION header
-            if hdr.class != 0b01 { return unsafe{ALPROTO_FAILED}; }
+            if hdr.class != BerClass::Application { return unsafe{ALPROTO_FAILED}; }
             // Tag number should be <= 30
-            if hdr.tag >= 30 { return unsafe{ALPROTO_FAILED}; }
+            if hdr.tag.0 >= 30 { return unsafe{ALPROTO_FAILED}; }
             // Kerberos messages contain sequences
             if rem.is_empty() || rem[0] != 0x30 { return unsafe{ALPROTO_FAILED}; }
             // Check kerberos version
@@ -477,7 +455,7 @@ pub extern "C" fn rs_krb5_probing_parser_tcp(_flow: *const Flow,
 {
     let slice = build_slice!(input,input_len as usize);
     if slice.len() <= 14 { return unsafe{ALPROTO_FAILED}; }
-    match be_u32(slice) {
+    match be_u32(slice) as IResult<&[u8],u32> {
         Ok((rem, record_mark)) => {
             // protocol implementations forbid very large requests
             if record_mark > 16384 { return unsafe{ALPROTO_FAILED}; }
@@ -500,10 +478,13 @@ pub extern "C" fn rs_krb5_parse_request(_flow: *const core::Flow,
                                        input: *const u8,
                                        input_len: u32,
                                        _data: *const std::os::raw::c_void,
-                                       _flags: u8) -> i32 {
+                                       _flags: u8) -> AppLayerResult {
     let buf = build_slice!(input,input_len as usize);
     let state = cast_pointer!(state,KRB5State);
-    state.parse(buf, STREAM_TOSERVER)
+    if state.parse(buf, STREAM_TOSERVER) < 0 {
+        return AppLayerResult::err();
+    }
+    AppLayerResult::ok()
 }
 
 #[no_mangle]
@@ -513,10 +494,13 @@ pub extern "C" fn rs_krb5_parse_response(_flow: *const core::Flow,
                                        input: *const u8,
                                        input_len: u32,
                                        _data: *const std::os::raw::c_void,
-                                       _flags: u8) -> i32 {
+                                       _flags: u8) -> AppLayerResult {
     let buf = build_slice!(input,input_len as usize);
     let state = cast_pointer!(state,KRB5State);
-    state.parse(buf, STREAM_TOCLIENT)
+    if state.parse(buf, STREAM_TOCLIENT) < 0 {
+        return AppLayerResult::err();
+    }
+    AppLayerResult::ok()
 }
 
 #[no_mangle]
@@ -526,12 +510,11 @@ pub extern "C" fn rs_krb5_parse_request_tcp(_flow: *const core::Flow,
                                        input: *const u8,
                                        input_len: u32,
                                        _data: *const std::os::raw::c_void,
-                                       _flags: u8) -> i32 {
+                                       _flags: u8) -> AppLayerResult {
     let buf = build_slice!(input,input_len as usize);
     let state = cast_pointer!(state,KRB5State);
 
     let mut v : Vec<u8>;
-    let mut status = 0;
     let tcp_buffer = match state.record_ts {
         0 => buf,
         _ => {
@@ -539,7 +522,7 @@ pub extern "C" fn rs_krb5_parse_request_tcp(_flow: *const core::Flow,
             if state.defrag_buf_ts.len() + buf.len() > 100000 {
                 SCLogDebug!("rs_krb5_parse_request_tcp: TCP buffer exploded {} {}",
                             state.defrag_buf_ts.len(), buf.len());
-                return 1;
+                return AppLayerResult::err();
             }
             v = state.defrag_buf_ts.split_off(0);
             v.extend_from_slice(buf);
@@ -549,35 +532,34 @@ pub extern "C" fn rs_krb5_parse_request_tcp(_flow: *const core::Flow,
     let mut cur_i = tcp_buffer;
     while cur_i.len() > 0 {
         if state.record_ts == 0 {
-            match be_u32(cur_i) {
+            match be_u32(cur_i) as IResult<&[u8],u32> {
                 Ok((rem,record)) => {
                     state.record_ts = record as usize;
                     cur_i = rem;
                 },
                 Err(nom::Err::Incomplete(_)) => {
                     state.defrag_buf_ts.extend_from_slice(cur_i);
-                    return 0;
+                    return AppLayerResult::ok();
                 }
                 _ => {
                     SCLogDebug!("rs_krb5_parse_request_tcp: reading record mark failed!");
-                    return 1;
+                    return AppLayerResult::err();
                 }
             }
         }
         if cur_i.len() >= state.record_ts {
-            status = state.parse(cur_i, STREAM_TOSERVER);
-            if status != 0 {
-                return status;
+            if state.parse(cur_i, STREAM_TOSERVER) < 0 {
+                return AppLayerResult::err();
             }
             state.record_ts = 0;
             cur_i = &cur_i[state.record_ts..];
         } else {
             // more fragments required
             state.defrag_buf_ts.extend_from_slice(cur_i);
-            return 0;
+            return AppLayerResult::ok();
         }
     }
-    status
+    AppLayerResult::ok()
 }
 
 #[no_mangle]
@@ -587,12 +569,11 @@ pub extern "C" fn rs_krb5_parse_response_tcp(_flow: *const core::Flow,
                                        input: *const u8,
                                        input_len: u32,
                                        _data: *const std::os::raw::c_void,
-                                       _flags: u8) -> i32 {
+                                       _flags: u8) -> AppLayerResult {
     let buf = build_slice!(input,input_len as usize);
     let state = cast_pointer!(state,KRB5State);
 
     let mut v : Vec<u8>;
-    let mut status = 0;
     let tcp_buffer = match state.record_tc {
         0 => buf,
         _ => {
@@ -600,7 +581,7 @@ pub extern "C" fn rs_krb5_parse_response_tcp(_flow: *const core::Flow,
             if state.defrag_buf_tc.len() + buf.len() > 100000 {
                 SCLogDebug!("rs_krb5_parse_response_tcp: TCP buffer exploded {} {}",
                             state.defrag_buf_tc.len(), buf.len());
-                return 1;
+                return AppLayerResult::err();
             }
             v = state.defrag_buf_tc.split_off(0);
             v.extend_from_slice(buf);
@@ -610,39 +591,37 @@ pub extern "C" fn rs_krb5_parse_response_tcp(_flow: *const core::Flow,
     let mut cur_i = tcp_buffer;
     while cur_i.len() > 0 {
         if state.record_tc == 0 {
-            match be_u32(cur_i) {
+            match be_u32(cur_i) as IResult<&[u8],_> {
                 Ok((rem,record)) => {
                     state.record_tc = record as usize;
                     cur_i = rem;
                 },
                 Err(nom::Err::Incomplete(_)) => {
                     state.defrag_buf_tc.extend_from_slice(cur_i);
-                    return 0;
+                    return AppLayerResult::ok();
                 }
                 _ => {
-                    SCLogNotice!("rs_krb5_parse_response_tcp: reading record mark failed!");
-                    return 1;
+                    SCLogDebug!("reading record mark failed!");
+                    return AppLayerResult::ok();
                 }
             }
         }
         if cur_i.len() >= state.record_tc {
-            status = state.parse(cur_i, STREAM_TOCLIENT);
-            if status != 0 {
-                return status;
+            if state.parse(cur_i, STREAM_TOCLIENT) < 0 {
+                return AppLayerResult::err();
             }
             state.record_tc = 0;
             cur_i = &cur_i[state.record_tc..];
         } else {
             // more fragments required
             state.defrag_buf_tc.extend_from_slice(cur_i);
-            return 0;
+            return AppLayerResult::ok();
         }
     }
-    status
+    AppLayerResult::ok()
 }
 
-export_tx_detect_flags_set!(rs_krb5_tx_detect_flags_set, KRB5Transaction);
-export_tx_detect_flags_get!(rs_krb5_tx_detect_flags_get, KRB5Transaction);
+export_tx_data_get!(rs_krb5_get_tx_data, KRB5Transaction);
 
 const PARSER_NAME : &'static [u8] = b"krb5\0";
 
@@ -653,8 +632,8 @@ pub unsafe extern "C" fn rs_register_krb5_parser() {
         name               : PARSER_NAME.as_ptr() as *const std::os::raw::c_char,
         default_port       : default_port.as_ptr(),
         ipproto            : core::IPPROTO_UDP,
-        probe_ts           : rs_krb5_probing_parser,
-        probe_tc           : rs_krb5_probing_parser,
+        probe_ts           : Some(rs_krb5_probing_parser),
+        probe_tc           : Some(rs_krb5_probing_parser),
         min_depth          : 0,
         max_depth          : 16,
         state_new          : rs_krb5_state_new,
@@ -666,8 +645,6 @@ pub unsafe extern "C" fn rs_register_krb5_parser() {
         get_tx             : rs_krb5_state_get_tx,
         tx_get_comp_st     : rs_krb5_state_progress_completion_status,
         tx_get_progress    : rs_krb5_tx_get_alstate_progress,
-        get_tx_logged      : Some(rs_krb5_tx_get_logged),
-        set_tx_logged      : Some(rs_krb5_tx_set_logged),
         get_de_state       : rs_krb5_state_get_tx_detect_state,
         set_de_state       : rs_krb5_state_set_tx_detect_state,
         get_events         : Some(rs_krb5_state_get_events),
@@ -675,12 +652,12 @@ pub unsafe extern "C" fn rs_register_krb5_parser() {
         get_eventinfo_byid : Some(rs_krb5_state_get_event_info_by_id),
         localstorage_new   : None,
         localstorage_free  : None,
-        get_tx_mpm_id      : None,
-        set_tx_mpm_id      : None,
         get_files          : None,
         get_tx_iterator    : None,
-        get_tx_detect_flags: Some(rs_krb5_tx_detect_flags_get),
-        set_tx_detect_flags: Some(rs_krb5_tx_detect_flags_set),
+        get_tx_data        : rs_krb5_get_tx_data,
+        apply_tx_config    : None,
+        flags              : APP_LAYER_PARSER_OPT_UNIDIR_TXS,
+        truncate           : None,
     };
     // register UDP parser
     let ip_proto_str = CString::new("udp").unwrap();
@@ -696,8 +673,8 @@ pub unsafe extern "C" fn rs_register_krb5_parser() {
     }
     // register TCP parser
     parser.ipproto = core::IPPROTO_TCP;
-    parser.probe_ts = rs_krb5_probing_parser_tcp;
-    parser.probe_tc = rs_krb5_probing_parser_tcp;
+    parser.probe_ts = Some(rs_krb5_probing_parser_tcp);
+    parser.probe_tc = Some(rs_krb5_probing_parser_tcp);
     parser.parse_ts = rs_krb5_parse_request_tcp;
     parser.parse_tc = rs_krb5_parse_response_tcp;
     let ip_proto_str = CString::new("tcp").unwrap();
