@@ -16,18 +16,17 @@
  */
 
 // written by Victor Julien
-// TCP buffering code written by Pierre Chifflier
 
 use std;
+use std::cmp;
 use std::mem::transmute;
 use std::collections::{HashMap};
 use std::ffi::CStr;
 
 use nom;
 
-use crate::log::*;
 use crate::applayer;
-use crate::applayer::LoggerFlags;
+use crate::applayer::{AppLayerResult, AppLayerTxData};
 use crate::core::*;
 use crate::filetracker::*;
 use crate::filecontainer::*;
@@ -179,12 +178,10 @@ pub struct NFSTransaction {
     /// attempt failed.
     pub type_data: Option<NFSTransactionTypeData>,
 
-    detect_flags_ts: u64,
-    detect_flags_tc: u64,
-
-    pub logged: LoggerFlags,
     pub de_state: Option<*mut DetectEngineState>,
     pub events: *mut AppLayerDecoderEvents,
+
+    pub tx_data: AppLayerTxData,
 }
 
 impl NFSTransaction {
@@ -209,11 +206,9 @@ impl NFSTransaction {
             file_tx_direction: 0,
             file_handle:Vec::new(),
             type_data: None,
-            detect_flags_ts: 0,
-            detect_flags_tc: 0,
-            logged: LoggerFlags::new(),
             de_state: None,
             events: std::ptr::null_mut(),
+            tx_data: AppLayerTxData::new(),
         }
     }
 
@@ -320,10 +315,6 @@ pub struct NFSState {
     /// transactions list
     pub transactions: Vec<NFSTransaction>,
 
-    /// TCP segments defragmentation buffer
-    pub tcp_buffer_ts: Vec<u8>,
-    pub tcp_buffer_tc: Vec<u8>,
-
     pub files: NFSFiles,
 
     /// partial record tracking
@@ -346,6 +337,7 @@ pub struct NFSState {
     /// true as long as we have file txs that are in a post-gap
     /// state. It means we'll do extra house keeping for those.
     check_post_gap_file_txs: bool,
+    post_gap_files_checked: bool,
 
     pub nfs_version: u16,
 
@@ -366,8 +358,6 @@ impl NFSState {
             requestmap:HashMap::new(),
             namemap:HashMap::new(),
             transactions: Vec::new(),
-            tcp_buffer_ts:Vec::with_capacity(8192),
-            tcp_buffer_tc:Vec::with_capacity(8192),
             files:NFSFiles::new(),
             ts_chunk_xid:0,
             tc_chunk_xid:0,
@@ -380,12 +370,21 @@ impl NFSState {
             tc_gap:false,
             is_udp:false,
             check_post_gap_file_txs:false,
+            post_gap_files_checked:false,
             nfs_version:0,
             events:0,
             tx_id:0,
             ts: 0,
         }
     }
+
+    fn update_ts(&mut self, ts: u64) {
+        if ts != self.ts {
+            self.ts = ts;
+            self.post_gap_files_checked = false;
+        }
+    }
+
     pub fn free(&mut self) {
         self.files.free();
     }
@@ -502,11 +501,13 @@ impl NFSState {
     {
         let mut post_gap_txs = false;
         for tx in &mut self.transactions {
-            if let Some(NFSTransactionTypeData::FILE(ref f)) = tx.type_data {
+            if let Some(NFSTransactionTypeData::FILE(ref mut f)) = tx.type_data {
                 if f.post_gap_ts > 0 {
                     if self.ts > f.post_gap_ts {
                         tx.request_done = true;
                         tx.response_done = true;
+                        let (files, flags) = self.files.get(tx.file_tx_direction);
+                        f.file_tracker.trunc(files, flags);
                     } else {
                         post_gap_txs = true;
                     }
@@ -589,7 +590,7 @@ impl NFSState {
     }
 
     /// complete request record
-    fn process_request_record<'b>(&mut self, r: &RpcPacket<'b>) -> u32 {
+    fn process_request_record<'b>(&mut self, r: &RpcPacket<'b>) {
         SCLogDebug!("REQUEST {} procedure {} ({}) blob size {}",
                 r.hdr.xid, r.procedure, self.requestmap.len(), r.prog_data.len());
 
@@ -603,7 +604,7 @@ impl NFSState {
             2 => {
                 self.process_request_record_v2(r)
             },
-            _ => { 1 },
+            _ => { },
         }
     }
 
@@ -657,15 +658,12 @@ impl NFSState {
         }
 
         let file_handle = w.handle.value.to_vec();
-        let file_name = match self.namemap.get(w.handle.value) {
-            Some(n) => {
-                SCLogDebug!("WRITE name {:?}", n);
-                n.to_vec()
-            },
-            None => {
-                SCLogDebug!("WRITE object {:?} not found", w.handle.value);
-                Vec::new()
-            },
+        let file_name = if let Some(name) = self.namemap.get(w.handle.value) {
+            SCLogDebug!("WRITE name {:?}", name);
+            name.to_vec()
+        } else {
+            SCLogDebug!("WRITE object {:?} not found", w.handle.value);
+            Vec::new()
         };
 
         let found = match self.get_file_tx_by_handle(&file_handle, STREAM_TOSERVER) {
@@ -747,15 +745,18 @@ impl NFSState {
         match xidmap.progver {
             2 => {
                 SCLogDebug!("NFSv2 reply record");
-                return self.process_reply_record_v2(r, &xidmap);
+                self.process_reply_record_v2(r, &xidmap);
+                return 0;
             },
             3 => {
                 SCLogDebug!("NFSv3 reply record");
-                return self.process_reply_record_v3(r, &mut xidmap);
+                self.process_reply_record_v3(r, &mut xidmap);
+                return 0;
             },
             4 => {
                 SCLogDebug!("NFSv4 reply record");
-                return self.process_reply_record_v4(r, &mut xidmap);
+                self.process_reply_record_v4(r, &mut xidmap);
+                return 0;
             },
             _ => {
                 SCLogDebug!("Invalid NFS version");
@@ -912,10 +913,8 @@ impl NFSState {
 
         if nfs_version == 2 {
             let size = match parse_nfs2_attribs(reply.attr_blob) {
-                Ok((_, ref attr)) => {
-                    attr.asize
-                },
-                _ => { 0 },
+                Ok((_, ref attr)) => attr.asize,
+                _ => 0,
             };
             SCLogDebug!("NFSv2 READ reply record: File size {}. Offset {} data len {}: total {}",
                     size, chunk_offset, reply.data_len, chunk_offset + reply.data_len as u64);
@@ -1003,82 +1002,57 @@ impl NFSState {
     }
 
     fn peek_reply_record(&mut self, r: &RpcPacketHeader) -> u32 {
-        let xidmap;
-        match self.requestmap.get(&r.xid) {
-            Some(p) => { xidmap = p; },
-            _ => { SCLogDebug!("REPLY: xid {} NOT FOUND", r.xid); return 0; },
+        if let Some(xidmap) = self.requestmap.get(&r.xid) {
+            return xidmap.procedure;
+        } else {
+            SCLogDebug!("REPLY: xid {} NOT FOUND", r.xid);
+            return 0;
         }
-
-        xidmap.procedure
     }
 
-    pub fn parse_tcp_data_ts_gap<'b>(&mut self, gap_size: u32) -> u32 {
+    pub fn parse_tcp_data_ts_gap<'b>(&mut self, gap_size: u32) -> AppLayerResult {
         SCLogDebug!("parse_tcp_data_ts_gap ({})", gap_size);
-        if self.tcp_buffer_ts.len() > 0 {
-            self.tcp_buffer_ts.clear();
-        }
         let gap = vec![0; gap_size as usize];
         let consumed = self.filetracker_update(STREAM_TOSERVER, &gap, gap_size);
         if consumed > gap_size {
             SCLogDebug!("consumed more than GAP size: {} > {}", consumed, gap_size);
-            return 1;
+            return AppLayerResult::ok();
         }
         self.ts_ssn_gap = true;
         self.ts_gap = true;
         SCLogDebug!("parse_tcp_data_ts_gap ({}) done", gap_size);
-        return 0
+        return AppLayerResult::ok();
     }
 
-    pub fn parse_tcp_data_tc_gap<'b>(&mut self, gap_size: u32) -> u32 {
+    pub fn parse_tcp_data_tc_gap<'b>(&mut self, gap_size: u32) -> AppLayerResult {
         SCLogDebug!("parse_tcp_data_tc_gap ({})", gap_size);
-        if self.tcp_buffer_tc.len() > 0 {
-            self.tcp_buffer_tc.clear();
-        }
         let gap = vec![0; gap_size as usize];
         let consumed = self.filetracker_update(STREAM_TOCLIENT, &gap, gap_size);
         if consumed > gap_size {
             SCLogDebug!("consumed more than GAP size: {} > {}", consumed, gap_size);
-            return 1;
+            return AppLayerResult::ok();
         }
         self.tc_ssn_gap = true;
         self.tc_gap = true;
         SCLogDebug!("parse_tcp_data_tc_gap ({}) done", gap_size);
-        return 0
+        return AppLayerResult::ok();
     }
 
     /// Parsing function, handling TCP chunks fragmentation
-    pub fn parse_tcp_data_ts<'b>(&mut self, i: &'b[u8]) -> u32 {
-        let mut v : Vec<u8>;
-        let mut status = 0;
-        SCLogDebug!("parse_tcp_data_ts ({})",i.len());
-        //SCLogDebug!("{:?}",i);
-        // Check if TCP data is being defragmented
-        let tcp_buffer = match self.tcp_buffer_ts.len() {
-            0 => i,
-            _ => {
-                v = self.tcp_buffer_ts.split_off(0);
-                // sanity check vector length to avoid memory exhaustion
-                if self.tcp_buffer_ts.len() + i.len() > 1000000 {
-                    SCLogDebug!("parse_tcp_data_ts: TS buffer exploded {} {}",
-                            self.tcp_buffer_ts.len(), i.len());
-                    return 1;
-                };
-                v.extend_from_slice(i);
-                v.as_slice()
-            },
-        };
-        //SCLogDebug!("tcp_buffer ({})",tcp_buffer.len());
-        let mut cur_i = tcp_buffer;
-        if cur_i.len() > 1000000 {
-            SCLogDebug!("BUG buffer exploded: {}", cur_i.len());
-        }
+    pub fn parse_tcp_data_ts<'b>(&mut self, i: &'b[u8]) -> AppLayerResult {
+        let mut cur_i = i;
         // take care of in progress file chunk transfers
         // and skip buffer beyond it
         let consumed = self.filetracker_update(STREAM_TOSERVER, cur_i, 0);
         if consumed > 0 {
-            if consumed > cur_i.len() as u32 { return 1; }
+            if consumed > cur_i.len() as u32 {
+                return AppLayerResult::err();
+            }
             cur_i = &cur_i[consumed as usize..];
         }
+        if cur_i.len() == 0 {
+            return AppLayerResult::ok();
+        }
         if self.ts_gap {
             SCLogDebug!("TS trying to catch up after GAP (input {})", cur_i.len());
 
@@ -1092,9 +1066,9 @@ impl NFSState {
                         break;
                     },
                     0 => {
-                        SCLogDebug!("incomplete, queue and retry with the next block (input {}). Looped {} times.", cur_i.len(), cnt);
-                        self.tcp_buffer_ts.extend_from_slice(cur_i);
-                        return 0;
+                        SCLogDebug!("incomplete, queue and retry with the next block (input {}). Looped {} times.",
+                                cur_i.len(), cnt);
+                        return AppLayerResult::incomplete((i.len() - cur_i.len()) as u32, (cur_i.len() + 1) as u32);
                     },
                     -1 => {
                         cur_i = &cur_i[1..];
@@ -1102,7 +1076,9 @@ impl NFSState {
                             SCLogDebug!("all post-GAP data in this chunk was bad. Looped {} times.", cnt);
                         }
                     },
-                    _ => { return 1; },
+                    _ => {
+                        return AppLayerResult::err();
+                    },
                 }
             }
             SCLogDebug!("TS GAP handling done (input {})", cur_i.len());
@@ -1132,7 +1108,7 @@ impl NFSState {
                                         match parse_nfs3_request_write(rpc_record.prog_data) {
                                             Ok((_, ref nfs_request_write)) => {
                                                 // deal with the partial nfs write data
-                                                status |= self.process_partial_write_request_record(rpc_record, nfs_request_write);
+                                                self.process_partial_write_request_record(rpc_record, nfs_request_write);
                                                 cur_i = remaining; // progress input past parsed record
                                             },
                                             _ => {
@@ -1146,18 +1122,22 @@ impl NFSState {
                                         // have Incomplete data. Fall through to the buffer code
                                         // and try again on our next iteration.
                                         SCLogDebug!("TS data incomplete");
+                                        // fall through to the incomplete below
                                     },
                                     Err(nom::Err::Error(_e)) |
                                     Err(nom::Err::Failure(_e)) => {
                                         self.set_event(NFSEvent::MalformedData);
                                         SCLogDebug!("Parsing failed: {:?}", _e);
-                                        return 1;
+                                        return AppLayerResult::err();
                                     },
                                 }
                             }
                         }
-                        self.tcp_buffer_ts.extend_from_slice(cur_i);
-                        break;
+                        // make sure we pass a value higher than current input
+                        // but lower than the record size
+                        let n1 = cmp::max(cur_i.len(), 1024);
+                        let n2 = cmp::min(n1, rec_size);
+                        return AppLayerResult::incomplete((i.len() - cur_i.len()) as u32, n2 as u32);
                     }
 
                     // we have the full records size worth of data,
@@ -1165,7 +1145,7 @@ impl NFSState {
                     match parse_rpc(&cur_i[..rec_size]) {
                         Ok((_, ref rpc_record)) => {
                             cur_i = &cur_i[rec_size..];
-                            status |= self.process_request_record(rpc_record);
+                            self.process_request_record(rpc_record);
                         },
                         Err(nom::Err::Incomplete(_)) => {
                             cur_i = &cur_i[rec_size..]; // progress input past parsed record
@@ -1174,74 +1154,58 @@ impl NFSState {
                             // so if we got incomplete anyway it's the data that is
                             // bad.
                             self.set_event(NFSEvent::MalformedData);
-
-                            status = 1;
                         },
                         Err(nom::Err::Error(_e)) |
                         Err(nom::Err::Failure(_e)) => {
                             self.set_event(NFSEvent::MalformedData);
                             SCLogDebug!("Parsing failed: {:?}", _e);
-                            return 1;
+                            return AppLayerResult::err();
                         },
                     }
                 },
-                Err(nom::Err::Incomplete(_)) => {
-                    SCLogDebug!("Fragmentation required (TCP level) 2");
-                    self.tcp_buffer_ts.extend_from_slice(cur_i);
-                    break;
+                Err(nom::Err::Incomplete(needed)) => {
+                    if let nom::Needed::Size(n) = needed {
+                        SCLogDebug!("Not enough data for partial RPC header {:?}", needed);
+                        // 28 is the partial RPC header size parse_rpc_request_partial
+                        // looks for.
+                        let need = if n > 28 { n } else { 28 };
+                        return AppLayerResult::incomplete((i.len() - cur_i.len()) as u32, need as u32);
+                    }
+                    return AppLayerResult::err();
                 },
                 Err(nom::Err::Error(_e)) |
                 Err(nom::Err::Failure(_e)) => {
                     self.set_event(NFSEvent::MalformedData);
                     SCLogDebug!("Parsing failed: {:?}", _e);
-                    return 1;
+                    return AppLayerResult::err();
                 },
             }
         };
 
         self.post_gap_housekeeping(STREAM_TOSERVER);
-        if self.check_post_gap_file_txs {
+        if self.check_post_gap_file_txs && !self.post_gap_files_checked {
             self.post_gap_housekeeping_for_files();
+            self.post_gap_files_checked = true;
         }
 
-        status
+        AppLayerResult::ok()
     }
 
     /// Parsing function, handling TCP chunks fragmentation
-    pub fn parse_tcp_data_tc<'b>(&mut self, i: &'b[u8]) -> u32 {
-        let mut v : Vec<u8>;
-        let mut status = 0;
-        SCLogDebug!("parse_tcp_data_tc ({})",i.len());
-        //SCLogDebug!("{:?}",i);
-        // Check if TCP data is being defragmented
-        let tcp_buffer = match self.tcp_buffer_tc.len() {
-            0 => i,
-            _ => {
-                v = self.tcp_buffer_tc.split_off(0);
-                // sanity check vector length to avoid memory exhaustion
-                if self.tcp_buffer_tc.len() + i.len() > 100000 {
-                    SCLogDebug!("TC buffer exploded");
-                    return 1;
-                };
-
-                v.extend_from_slice(i);
-                v.as_slice()
-            },
-        };
-        SCLogDebug!("TC tcp_buffer ({}), input ({})",tcp_buffer.len(), i.len());
-
-        let mut cur_i = tcp_buffer;
-        if cur_i.len() > 100000 {
-            SCLogDebug!("parse_tcp_data_tc: BUG buffer exploded {}", cur_i.len());
-        }
-
+    pub fn parse_tcp_data_tc<'b>(&mut self, i: &'b[u8]) -> AppLayerResult {
+        let mut cur_i = i;
         // take care of in progress file chunk transfers
         // and skip buffer beyond it
         let consumed = self.filetracker_update(STREAM_TOCLIENT, cur_i, 0);
         if consumed > 0 {
-            if consumed > cur_i.len() as u32 { return 1; }
+            if consumed > cur_i.len() as u32 {
+                return AppLayerResult::err();
+            }
             cur_i = &cur_i[consumed as usize..];
         }
+        if cur_i.len() == 0 {
+            return AppLayerResult::ok();
+        }
         if self.tc_gap {
             SCLogDebug!("TC trying to catch up after GAP (input {})", cur_i.len());
 
@@ -1255,9 +1219,9 @@ impl NFSState {
                         break;
                     },
                     0 => {
-                        SCLogDebug!("incomplete, queue and retry with the next block (input {}). Looped {} times.", cur_i.len(), cnt);
-                        self.tcp_buffer_tc.extend_from_slice(cur_i);
-                        return 0;
+                        SCLogDebug!("incomplete, queue and retry with the next block (input {}). Looped {} times.",
+                                cur_i.len(), cnt);
+                        return AppLayerResult::incomplete((i.len() - cur_i.len()) as u32, (cur_i.len() + 1) as u32);
                     },
                     -1 => {
                         cur_i = &cur_i[1..];
@@ -1265,7 +1229,9 @@ impl NFSState {
                             SCLogDebug!("all post-GAP data in this chunk was bad. Looped {} times.", cnt);
                         }
                     },
-                    _ => { return 1; },
+                    _ => {
+                        return AppLayerResult::err();
+                    }
                 }
             }
             SCLogDebug!("TC GAP handling done (input {})", cur_i.len());
@@ -1293,17 +1259,16 @@ impl NFSState {
                                         match parse_nfs3_reply_read(rpc_record.prog_data) {
                                             Ok((_, ref nfs_reply_read)) => {
                                                 // deal with the partial nfs read data
-                                                status |= self.process_partial_read_reply_record(rpc_record, nfs_reply_read);
+                                                self.process_partial_read_reply_record(rpc_record, nfs_reply_read);
                                                 cur_i = remaining; // progress input past parsed record
                                             },
                                             Err(nom::Err::Incomplete(_)) => {
-                                                self.set_event(NFSEvent::MalformedData);
                                             },
                                             Err(nom::Err::Error(_e)) |
                                             Err(nom::Err::Failure(_e)) => {
                                                 self.set_event(NFSEvent::MalformedData);
                                                 SCLogDebug!("Parsing failed: {:?}", _e);
-                                                return 1;
+                                                return AppLayerResult::err();
                                             }
                                         }
                                     },
@@ -1316,20 +1281,23 @@ impl NFSState {
                                     Err(nom::Err::Failure(_e)) => {
                                         self.set_event(NFSEvent::MalformedData);
                                         SCLogDebug!("Parsing failed: {:?}", _e);
-                                        return 1;
+                                        return AppLayerResult::err();
                                     }
                                 }
                             }
                         }
-                        self.tcp_buffer_tc.extend_from_slice(cur_i);
-                        break;
+                        // make sure we pass a value higher than current input
+                        // but lower than the record size
+                        let n1 = cmp::max(cur_i.len(), 1024);
+                        let n2 = cmp::min(n1, rec_size);
+                        return AppLayerResult::incomplete((i.len() - cur_i.len()) as u32, n2 as u32);
                     }
 
                     // we have the full data of the record, lets parse
                     match parse_rpc_reply(&cur_i[..rec_size]) {
                         Ok((_, ref rpc_record)) => {
                             cur_i = &cur_i[rec_size..]; // progress input past parsed record
-                            status |= self.process_reply_record(rpc_record);
+                            self.process_reply_record(rpc_record);
                         },
                         Err(nom::Err::Incomplete(_)) => {
                             cur_i = &cur_i[rec_size..]; // progress input past parsed record
@@ -1338,39 +1306,42 @@ impl NFSState {
                             // so if we got incomplete anyway it's the data that is
                             // bad.
                             self.set_event(NFSEvent::MalformedData);
-
-                            status = 1;
                         },
                         Err(nom::Err::Error(_e)) |
                         Err(nom::Err::Failure(_e)) => {
                             self.set_event(NFSEvent::MalformedData);
                             SCLogDebug!("Parsing failed: {:?}", _e);
-                            return 1;
+                            return AppLayerResult::err();
                         }
                     }
                 },
-                Err(nom::Err::Incomplete(_)) => {
-                    SCLogDebug!("REPLY: insufficient data for HDR");
-                    self.tcp_buffer_tc.extend_from_slice(cur_i);
-                    break;
+                Err(nom::Err::Incomplete(needed)) => {
+                    if let nom::Needed::Size(n) = needed {
+                        SCLogDebug!("Not enough data for partial RPC header {:?}", needed);
+                        // 12 is the partial RPC header size parse_rpc_packet_header
+                        // looks for.
+                        let need = if n > 12 { n } else { 12 };
+                        return AppLayerResult::incomplete((i.len() - cur_i.len()) as u32, need as u32);
+                    }
+                    return AppLayerResult::err();
                 },
                 Err(nom::Err::Error(_e)) |
                 Err(nom::Err::Failure(_e)) => {
                     self.set_event(NFSEvent::MalformedData);
                     SCLogDebug!("Parsing failed: {:?}", _e);
-                    return 1;
+                    return AppLayerResult::err();
                 },
             }
         };
         self.post_gap_housekeeping(STREAM_TOCLIENT);
-        if self.check_post_gap_file_txs {
+        if self.check_post_gap_file_txs && !self.post_gap_files_checked {
             self.post_gap_housekeeping_for_files();
+            self.post_gap_files_checked = true;
         }
-        status
+        AppLayerResult::ok()
     }
     /// Parsing function
-    pub fn parse_udp_ts<'b>(&mut self, input: &'b[u8]) -> u32 {
-        let mut status = 0;
+    pub fn parse_udp_ts<'b>(&mut self, input: &'b[u8]) -> AppLayerResult {
         SCLogDebug!("parse_udp_ts ({})", input.len());
         if input.len() > 0 {
             match parse_rpc_udp_request(input) {
@@ -1378,41 +1349,45 @@ impl NFSState {
                     self.is_udp = true;
                     match rpc_record.progver {
                         3 => {
-                            status |= self.process_request_record(rpc_record);
+                            self.process_request_record(rpc_record);
                         },
                         2 => {
-                            status |= self.process_request_record_v2(rpc_record);
+                            self.process_request_record_v2(rpc_record);
                         },
-                        _ => { status = 1; },
+                        _ => { },
                     }
                 },
                 Err(nom::Err::Incomplete(_)) => {
                 },
                 Err(nom::Err::Error(_e)) |
-                Err(nom::Err::Failure(_e)) => { SCLogDebug!("Parsing failed: {:?}", _e); }
+                Err(nom::Err::Failure(_e)) => {
+                    SCLogDebug!("Parsing failed: {:?}", _e);
+                }
             }
         }
-        status
+        AppLayerResult::ok()
     }
 
     /// Parsing function
-    pub fn parse_udp_tc<'b>(&mut self, input: &'b[u8]) -> u32 {
-        let mut status = 0;
+    pub fn parse_udp_tc<'b>(&mut self, input: &'b[u8]) -> AppLayerResult {
         SCLogDebug!("parse_udp_tc ({})", input.len());
         if input.len() > 0 {
             match parse_rpc_udp_reply(input) {
                 Ok((_, ref rpc_record)) => {
                     self.is_udp = true;
-                    status |= self.process_reply_record(rpc_record);
+                    self.process_reply_record(rpc_record);
                 },
                 Err(nom::Err::Incomplete(_)) => {
                 },
                 Err(nom::Err::Error(_e)) |
-                Err(nom::Err::Failure(_e)) => { SCLogDebug!("Parsing failed: {:?}", _e); }
+                Err(nom::Err::Failure(_e)) => {
+                    SCLogDebug!("Parsing failed: {:?}", _e);
+                }
             }
-        };
-        status
+        }
+        AppLayerResult::ok()
     }
+
     fn getfiles(&mut self, direction: u8) -> * mut FileContainer {
         //SCLogDebug!("direction: {}", direction);
         if direction == STREAM_TOCLIENT {
@@ -1433,7 +1408,7 @@ impl NFSState {
 
 /// Returns *mut NFSState
 #[no_mangle]
-pub extern "C" fn rs_nfs_state_new() -> *mut std::os::raw::c_void {
+pub extern "C" fn rs_nfs_state_new(_orig_state: *mut std::os::raw::c_void, _orig_proto: AppProto) -> *mut std::os::raw::c_void {
     let state = NFSState::new();
     let boxed = Box::new(state);
     SCLogDebug!("allocating state");
@@ -1458,29 +1433,22 @@ pub extern "C" fn rs_nfs_parse_request(flow: &mut Flow,
                                        input: *const u8,
                                        input_len: u32,
                                        _data: *mut std::os::raw::c_void)
-                                       -> i8
+                                       -> AppLayerResult
 {
     let buf = unsafe{std::slice::from_raw_parts(input, input_len as usize)};
     SCLogDebug!("parsing {} bytes of request data", input_len);
 
-    state.ts = flow.get_last_time().as_secs();
-    if state.parse_tcp_data_ts(buf) == 0 {
-        1
-    } else {
-        -1
-    }
+    state.update_ts(flow.get_last_time().as_secs());
+    state.parse_tcp_data_ts(buf)
 }
 
 #[no_mangle]
 pub extern "C" fn rs_nfs_parse_request_tcp_gap(
                                         state: &mut NFSState,
                                         input_len: u32)
-                                        -> i8
+                                        -> AppLayerResult
 {
-    if state.parse_tcp_data_ts_gap(input_len as u32) == 0 {
-        return 1;
-    }
-    return -1;
+    state.parse_tcp_data_ts_gap(input_len as u32)
 }
 
 #[no_mangle]
@@ -1490,29 +1458,22 @@ pub extern "C" fn rs_nfs_parse_response(flow: &mut Flow,
                                         input: *const u8,
                                         input_len: u32,
                                         _data: *mut std::os::raw::c_void)
-                                        -> i8
+                                        -> AppLayerResult
 {
     SCLogDebug!("parsing {} bytes of response data", input_len);
     let buf = unsafe{std::slice::from_raw_parts(input, input_len as usize)};
 
-    state.ts = flow.get_last_time().as_secs();
-    if state.parse_tcp_data_tc(buf) == 0 {
-        1
-    } else {
-        -1
-    }
+    state.update_ts(flow.get_last_time().as_secs());
+    state.parse_tcp_data_tc(buf)
 }
 
 #[no_mangle]
 pub extern "C" fn rs_nfs_parse_response_tcp_gap(
                                         state: &mut NFSState,
                                         input_len: u32)
-                                        -> i8
+                                        -> AppLayerResult
 {
-    if state.parse_tcp_data_tc_gap(input_len as u32) == 0 {
-        return 1;
-    }
-    return -1;
+    state.parse_tcp_data_tc_gap(input_len as u32)
 }
 
 /// C binding parse a DNS request. Returns 1 on success, -1 on failure.
@@ -1523,16 +1484,11 @@ pub extern "C" fn rs_nfs_parse_request_udp(_flow: *mut Flow,
                                        input: *const u8,
                                        input_len: u32,
                                        _data: *mut std::os::raw::c_void)
-                                       -> i8
+                                       -> AppLayerResult
 {
     let buf = unsafe{std::slice::from_raw_parts(input, input_len as usize)};
     SCLogDebug!("parsing {} bytes of request data", input_len);
-
-    if state.parse_udp_ts(buf) == 0 {
-        1
-    } else {
-        -1
-    }
+    state.parse_udp_ts(buf)
 }
 
 #[no_mangle]
@@ -1542,16 +1498,11 @@ pub extern "C" fn rs_nfs_parse_response_udp(_flow: *mut Flow,
                                         input: *const u8,
                                         input_len: u32,
                                         _data: *mut std::os::raw::c_void)
-                                        -> i8
+                                        -> AppLayerResult
 {
     SCLogDebug!("parsing {} bytes of response data", input_len);
     let buf = unsafe{std::slice::from_raw_parts(input, input_len as usize)};
-
-    if state.parse_udp_tc(buf) == 0 {
-        1
-    } else {
-        -1
-    }
+    state.parse_udp_tc(buf)
 }
 
 #[no_mangle]
@@ -1630,19 +1581,12 @@ pub extern "C" fn rs_nfs_tx_get_alstate_progress(tx: &mut NFSTransaction,
 }
 
 #[no_mangle]
-pub extern "C" fn rs_nfs_tx_set_logged(_state: &mut NFSState,
-                                       tx: &mut NFSTransaction,
-                                       logged: u32)
+pub extern "C" fn rs_nfs_get_tx_data(
+    tx: *mut std::os::raw::c_void)
+    -> *mut AppLayerTxData
 {
-    tx.logged.set(logged);
-}
-
-#[no_mangle]
-pub extern "C" fn rs_nfs_tx_get_logged(_state: &mut NFSState,
-                                       tx: &mut NFSTransaction)
-                                       -> u32
-{
-    return tx.logged.get();
+    let tx = cast_pointer!(tx, NFSTransaction);
+    return &mut tx.tx_data;
 }
 
 #[no_mangle]
@@ -1670,32 +1614,6 @@ pub extern "C" fn rs_nfs_state_get_tx_detect_state(
     }
 }
 
-#[no_mangle]
-pub extern "C" fn rs_nfs_tx_set_detect_flags(
-                                       tx: &mut NFSTransaction,
-                                       direction: u8,
-                                       flags: u64)
-{
-    if (direction & STREAM_TOSERVER) != 0 {
-        tx.detect_flags_ts = flags as u64;
-    } else {
-        tx.detect_flags_tc = flags as u64;
-    }
-}
-
-#[no_mangle]
-pub extern "C" fn rs_nfs_tx_get_detect_flags(
-                                       tx: &mut NFSTransaction,
-                                       direction: u8)
-                                       -> u64
-{
-    if (direction & STREAM_TOSERVER) != 0 {
-        return tx.detect_flags_ts as u64;
-    } else {
-        return tx.detect_flags_tc as u64;
-    }
-}
-
 #[no_mangle]
 pub extern "C" fn rs_nfs_state_get_events(tx: *mut std::os::raw::c_void)
                                           -> *mut AppLayerDecoderEvents
@@ -1844,14 +1762,13 @@ pub fn nfs_probe(i: &[u8], direction: u8) -> i8 {
                             return -1;
                         }
                     },
-                    Err(nom::Err::Incomplete(_)) => { },
+                    Err(nom::Err::Incomplete(_)) => {
+                        return 0;
+                    },
                     Err(_) => {
                         return -1;
                     },
                 }
-
-
-                return 0;
             },
             Err(_) => {
                 return -1;
