@@ -19,18 +19,13 @@
 
 //! RDP application layer
 
-use core::{
-    self, AppProto, DetectEngineState, Flow, ALPROTO_UNKNOWN, IPPROTO_TCP,
-};
-use conf;
+use crate::applayer::*;
+use crate::core::{self, AppProto, DetectEngineState, Flow, ALPROTO_UNKNOWN, IPPROTO_TCP};
+use crate::rdp::parser::*;
 use nom;
-use parser::*;
-use rdp::parser::*;
 use std;
 use std::mem::transmute;
-use tls_parser::{
-    parse_tls_plaintext, TlsMessage, TlsMessageHandshake, TlsRecordType,
-};
+use tls_parser::{parse_tls_plaintext, TlsMessage, TlsMessageHandshake, TlsRecordType};
 
 static mut ALPROTO_RDP: AppProto = ALPROTO_UNKNOWN;
 
@@ -58,6 +53,7 @@ pub struct RdpTransaction {
     pub item: RdpTransactionItem,
     // managed by macros `export_tx_get_detect_state!` and `export_tx_set_detect_state!`
     de_state: Option<*mut DetectEngineState>,
+    tx_data: AppLayerTxData,
 }
 
 impl RdpTransaction {
@@ -66,6 +62,7 @@ impl RdpTransaction {
             id,
             item,
             de_state: None,
+            tx_data: AppLayerTxData::new(),
         }
     }
 
@@ -84,8 +81,7 @@ impl Drop for RdpTransaction {
 
 #[no_mangle]
 pub extern "C" fn rs_rdp_state_get_tx(
-    state: *mut std::os::raw::c_void,
-    tx_id: u64,
+    state: *mut std::os::raw::c_void, tx_id: u64,
 ) -> *mut std::os::raw::c_void {
     let state = cast_pointer!(state, RdpState);
     match state.get_tx(tx_id) {
@@ -99,25 +95,20 @@ pub extern "C" fn rs_rdp_state_get_tx(
 }
 
 #[no_mangle]
-pub extern "C" fn rs_rdp_state_get_tx_count(
-    state: *mut std::os::raw::c_void,
-) -> u64 {
+pub extern "C" fn rs_rdp_state_get_tx_count(state: *mut std::os::raw::c_void) -> u64 {
     let state = cast_pointer!(state, RdpState);
     return state.next_id;
 }
 
 #[no_mangle]
-pub extern "C" fn rs_rdp_tx_get_progress_complete(
-    _direction: u8,
-) -> std::os::raw::c_int {
+pub extern "C" fn rs_rdp_tx_get_progress_complete(_direction: u8) -> std::os::raw::c_int {
     // a parser can implement a multi-step tx completion by using an arbitrary `n`
     return 1;
 }
 
 #[no_mangle]
 pub extern "C" fn rs_rdp_tx_get_progress(
-    _tx: *mut std::os::raw::c_void,
-    _direction: u8,
+    _tx: *mut std::os::raw::c_void, _direction: u8,
 ) -> std::os::raw::c_int {
     // tx complete when `rs_rdp_tx_get_progress(...) == rs_rdp_tx_get_progress_complete(...)`
     // here, all transactions are immediately complete on insert
@@ -131,8 +122,6 @@ pub extern "C" fn rs_rdp_tx_get_progress(
 #[derive(Debug, PartialEq)]
 pub struct RdpState {
     next_id: u64,
-    to_client: Vec<u8>,
-    to_server: Vec<u8>,
     transactions: Vec<RdpTransaction>,
     tls_parsing: bool,
     bypass_parsing: bool,
@@ -142,8 +131,6 @@ impl RdpState {
     fn new() -> Self {
         Self {
             next_id: 0,
-            to_client: Vec::new(),
-            to_server: Vec::new(),
             transactions: Vec::new(),
             tls_parsing: false,
             bypass_parsing: false,
@@ -183,35 +170,34 @@ impl RdpState {
     }
 
     /// parse buffer captures from client to server
-    fn parse_ts(&mut self, input: &[u8]) -> bool {
+    fn parse_ts(&mut self, input: &[u8]) -> AppLayerResult {
         // no need to process input buffer
         if self.bypass_parsing {
-            return true;
+            return AppLayerResult::ok();
         }
-        // combine residual buffer with provided buffer
-        self.to_server.extend(input);
-        let temp: Vec<u8> = self.to_server.split_off(0);
-        let mut available = temp.as_slice();
+        let mut available = input;
 
         loop {
             if available.len() == 0 {
-                return true;
+                return AppLayerResult::ok();
             }
             if self.tls_parsing {
                 match parse_tls_plaintext(&available) {
                     Ok((remainder, _tls)) => {
-                        // update awaiting-parsing buffer
+                        // bytes available for futher parsing are what remain
                         available = remainder;
                     }
 
                     Err(nom::Err::Incomplete(_)) => {
-                        // save unparsed residual buffer for next parse
-                        self.to_server.extend(available);
-                        return true;
+                        // nom need not compatible with applayer need, request one more byte
+                        return AppLayerResult::incomplete(
+                            (input.len() - available.len()) as u32,
+                            (available.len() + 1) as u32,
+                        );
                     }
 
                     Err(nom::Err::Failure(_)) | Err(nom::Err::Error(_)) => {
-                        return false;
+                        return AppLayerResult::err();
                     }
                 }
             } else {
@@ -219,17 +205,14 @@ impl RdpState {
                 match parse_t123_tpkt(&available) {
                     // success
                     Ok((remainder, t123)) => {
-                        // update awaiting-parsing buffer
+                        // bytes available for futher parsing are what remain
                         available = remainder;
                         // evaluate message within the tpkt
                         match t123.child {
                             // X.224 connection request
                             T123TpktChild::X224ConnectionRequest(x224) => {
-                                let tx = self.new_tx(
-                                    RdpTransactionItem::X224ConnectionRequest(
-                                        x224,
-                                    ),
-                                );
+                                let tx =
+                                    self.new_tx(RdpTransactionItem::X224ConnectionRequest(x224));
                                 self.transactions.push(tx);
                             }
 
@@ -252,17 +235,26 @@ impl RdpState {
                     }
 
                     Err(nom::Err::Incomplete(_)) => {
-                        // save unparsed residual buffer for next parse
-                        self.to_server.extend(available);
-                        return true;
+                        // nom need not compatible with applayer need, request one more byte
+                        return AppLayerResult::incomplete(
+                            (input.len() - available.len()) as u32,
+                            (available.len() + 1) as u32,
+                        );
                     }
 
                     Err(nom::Err::Failure(_)) | Err(nom::Err::Error(_)) => {
                         if probe_tls_handshake(available) {
                             self.tls_parsing = true;
-                            return self.parse_ts(available);
+                            let r = self.parse_ts(available);
+                            if r.status == 1 {
+                                //adds bytes already consumed to incomplete result
+                                let consumed = (input.len() - available.len()) as u32;
+                                return AppLayerResult::incomplete(r.consumed + consumed, r.needed);
+                            } else {
+                                return r;
+                            }
                         } else {
-                            return false;
+                            return AppLayerResult::err();
                         }
                     }
                 }
@@ -271,41 +263,35 @@ impl RdpState {
     }
 
     /// parse buffer captures from server to client
-    fn parse_tc(&mut self, input: &[u8]) -> bool {
+    fn parse_tc(&mut self, input: &[u8]) -> AppLayerResult {
         // no need to process input buffer
         if self.bypass_parsing {
-            return true;
+            return AppLayerResult::ok();
         }
-        // combine residual buffer with provided buffer
-        self.to_client.extend(input);
-        let temp: Vec<u8> = self.to_client.split_off(0);
-        let mut available = temp.as_slice();
+        let mut available = input;
 
         loop {
             if available.len() == 0 {
-                return true;
+                return AppLayerResult::ok();
             }
             if self.tls_parsing {
                 match parse_tls_plaintext(&available) {
                     Ok((remainder, tls)) => {
-                        // update awaiting-parsing buffer
+                        // bytes available for futher parsing are what remain
                         available = remainder;
                         for message in &tls.msg {
                             match message {
-                                TlsMessage::Handshake(
-                                    TlsMessageHandshake::Certificate(contents),
-                                ) => {
+                                TlsMessage::Handshake(TlsMessageHandshake::Certificate(
+                                    contents,
+                                )) => {
                                     let mut chain = Vec::new();
                                     for cert in &contents.cert_chain {
                                         chain.push(CertificateBlob {
                                             data: cert.data.to_vec(),
                                         });
                                     }
-                                    let tx = self.new_tx(
-                                        RdpTransactionItem::TlsCertificateChain(
-                                            chain,
-                                        ),
-                                    );
+                                    let tx =
+                                        self.new_tx(RdpTransactionItem::TlsCertificateChain(chain));
                                     self.transactions.push(tx);
                                     self.bypass_parsing = true;
                                 }
@@ -315,13 +301,15 @@ impl RdpState {
                     }
 
                     Err(nom::Err::Incomplete(_)) => {
-                        // save unparsed residual buffer for next parse
-                        self.to_client.extend(available);
-                        return true;
+                        // nom need not compatible with applayer need, request one more byte
+                        return AppLayerResult::incomplete(
+                            (input.len() - available.len()) as u32,
+                            (available.len() + 1) as u32,
+                        );
                     }
 
                     Err(nom::Err::Failure(_)) | Err(nom::Err::Error(_)) => {
-                        return false;
+                        return AppLayerResult::err();
                     }
                 }
             } else {
@@ -329,17 +317,14 @@ impl RdpState {
                 match parse_t123_tpkt(&available) {
                     // success
                     Ok((remainder, t123)) => {
-                        // update awaiting-parsing buffer
+                        // bytes available for futher parsing are what remain
                         available = remainder;
                         // evaluate message within the tpkt
                         match t123.child {
                             // X.224 connection confirm
                             T123TpktChild::X224ConnectionConfirm(x224) => {
-                                let tx = self.new_tx(
-                                    RdpTransactionItem::X224ConnectionConfirm(
-                                        x224,
-                                    ),
-                                );
+                                let tx =
+                                    self.new_tx(RdpTransactionItem::X224ConnectionConfirm(x224));
                                 self.transactions.push(tx);
                             }
 
@@ -351,7 +336,7 @@ impl RdpState {
                                             .new_tx(RdpTransactionItem::McsConnectResponse(mcs));
                                         self.transactions.push(tx);
                                         self.bypass_parsing = true;
-                                        return true;
+                                        return AppLayerResult::ok();
                                     }
 
                                     // unknown message in X.223, skip
@@ -365,8 +350,11 @@ impl RdpState {
                     }
 
                     Err(nom::Err::Incomplete(_)) => {
-                        self.to_client.extend(available);
-                        return true;
+                        // nom need not compatible with applayer need, request one more byte
+                        return AppLayerResult::incomplete(
+                            (input.len() - available.len()) as u32,
+                            (available.len() + 1) as u32,
+                        );
                     }
 
                     Err(nom::Err::Failure(_)) | Err(nom::Err::Error(_)) => {
@@ -374,7 +362,7 @@ impl RdpState {
                             self.tls_parsing = true;
                             return self.parse_tc(available);
                         } else {
-                            return false;
+                            return AppLayerResult::err();
                         }
                     }
                 }
@@ -384,7 +372,7 @@ impl RdpState {
 }
 
 #[no_mangle]
-pub extern "C" fn rs_rdp_state_new() -> *mut std::os::raw::c_void {
+pub extern "C" fn rs_rdp_state_new(_orig_state: *mut std::os::raw::c_void, _orig_proto: AppProto) -> *mut std::os::raw::c_void {
     let state = RdpState::new();
     let boxed = Box::new(state);
     return unsafe { std::mem::transmute(boxed) };
@@ -396,10 +384,7 @@ pub extern "C" fn rs_rdp_state_free(state: *mut std::os::raw::c_void) {
 }
 
 #[no_mangle]
-pub extern "C" fn rs_rdp_state_tx_free(
-    state: *mut std::os::raw::c_void,
-    tx_id: u64,
-) {
+pub extern "C" fn rs_rdp_state_tx_free(state: *mut std::os::raw::c_void, tx_id: u64) {
     let state = cast_pointer!(state, RdpState);
     state.free_tx(tx_id);
 }
@@ -423,11 +408,7 @@ fn probe_rdp(input: &[u8]) -> bool {
 /// probe for T.123 message, whether to client or to server
 #[no_mangle]
 pub extern "C" fn rs_rdp_probe_ts_tc(
-    _flow: *const Flow,
-    _direction: u8,
-    input: *const u8,
-    input_len: u32,
-    _rdir: *mut u8,
+    _flow: *const Flow, _direction: u8, input: *const u8, input_len: u32, _rdir: *mut u8,
 ) -> AppProto {
     if input != std::ptr::null_mut() {
         // probe bytes for `rdp` protocol pattern
@@ -454,44 +435,28 @@ fn probe_tls_handshake(input: &[u8]) -> bool {
 
 #[no_mangle]
 pub extern "C" fn rs_rdp_parse_ts(
-    _flow: *const Flow,
-    state: *mut std::os::raw::c_void,
-    _pstate: *mut std::os::raw::c_void,
-    input: *const u8,
-    input_len: u32,
-    _data: *const std::os::raw::c_void,
-    _flags: u8,
-) -> i32 {
+    _flow: *const Flow, state: *mut std::os::raw::c_void, _pstate: *mut std::os::raw::c_void,
+    input: *const u8, input_len: u32, _data: *const std::os::raw::c_void, _flags: u8,
+) -> AppLayerResult {
     let state = cast_pointer!(state, RdpState);
     let buf = build_slice!(input, input_len as usize);
     // attempt to parse bytes as `rdp` protocol
-    if state.parse_ts(buf) {
-        return 1;
-    }
-    // no need for further parsing
-    return -1;
+    return state.parse_ts(buf);
 }
 
 #[no_mangle]
 pub extern "C" fn rs_rdp_parse_tc(
-    _flow: *const Flow,
-    state: *mut std::os::raw::c_void,
-    _pstate: *mut std::os::raw::c_void,
-    input: *const u8,
-    input_len: u32,
-    _data: *const std::os::raw::c_void,
-    _flags: u8,
-) -> i32 {
+    _flow: *const Flow, state: *mut std::os::raw::c_void, _pstate: *mut std::os::raw::c_void,
+    input: *const u8, input_len: u32, _data: *const std::os::raw::c_void, _flags: u8,
+) -> AppLayerResult {
     let state = cast_pointer!(state, RdpState);
     let buf = build_slice!(input, input_len as usize);
     // attempt to parse bytes as `rdp` protocol
-    if state.parse_tc(buf) {
-        return 1;
-    }
-    // no need for further parsing
-    return -1;
+    return state.parse_tc(buf);
 }
 
+export_tx_data_get!(rs_rdp_get_tx_data, RdpTransaction);
+
 //
 // registration
 //
@@ -505,8 +470,8 @@ pub unsafe extern "C" fn rs_rdp_register_parser() {
         name: PARSER_NAME.as_ptr() as *const std::os::raw::c_char,
         default_port: default_port.as_ptr(),
         ipproto: IPPROTO_TCP,
-        probe_ts: rs_rdp_probe_ts_tc,
-        probe_tc: rs_rdp_probe_ts_tc,
+        probe_ts: Some(rs_rdp_probe_ts_tc),
+        probe_tc: Some(rs_rdp_probe_ts_tc),
         min_depth: 0,
         max_depth: 16,
         state_new: rs_rdp_state_new,
@@ -518,8 +483,6 @@ pub unsafe extern "C" fn rs_rdp_register_parser() {
         get_tx: rs_rdp_state_get_tx,
         tx_get_comp_st: rs_rdp_tx_get_progress_complete,
         tx_get_progress: rs_rdp_tx_get_progress,
-        get_tx_logged: None,
-        set_tx_logged: None,
         get_de_state: rs_rdp_tx_get_detect_state,
         set_de_state: rs_rdp_tx_set_detect_state,
         get_events: None,
@@ -527,32 +490,20 @@ pub unsafe extern "C" fn rs_rdp_register_parser() {
         get_eventinfo_byid: None,
         localstorage_new: None,
         localstorage_free: None,
-        get_tx_mpm_id: None,
-        set_tx_mpm_id: None,
         get_files: None,
         get_tx_iterator: None,
-        get_tx_detect_flags: None,
-        set_tx_detect_flags: None,
+        get_tx_data: rs_rdp_get_tx_data,
+        apply_tx_config: None,
+        flags: APP_LAYER_PARSER_OPT_UNIDIR_TXS,
+        truncate: None,
     };
 
-    /* For 5.0 we want this disabled by default, so check that it
-     * has been explicitly enabled. */
-    if !conf::conf_get_bool("app-layer.protocols.rdp.enabled") {
-        return;
-    }
-
     let ip_proto_str = std::ffi::CString::new("tcp").unwrap();
 
-    if AppLayerProtoDetectConfProtoDetectionEnabled(
-        ip_proto_str.as_ptr(),
-        parser.name,
-    ) != 0
-    {
+    if AppLayerProtoDetectConfProtoDetectionEnabled(ip_proto_str.as_ptr(), parser.name) != 0 {
         let alproto = AppLayerRegisterProtocolDetection(&parser, 1);
         ALPROTO_RDP = alproto;
-        if AppLayerParserConfParserEnabled(ip_proto_str.as_ptr(), parser.name)
-            != 0
-        {
+        if AppLayerParserConfParserEnabled(ip_proto_str.as_ptr(), parser.name) != 0 {
             let _ = AppLayerRegisterParser(&parser, alproto);
         }
     }
@@ -561,7 +512,7 @@ pub unsafe extern "C" fn rs_rdp_register_parser() {
 #[cfg(test)]
 mod tests {
     use super::*;
-    use rdp::parser::{RdpCookie, X224ConnectionRequest};
+    use crate::rdp::parser::{RdpCookie, X224ConnectionRequest};
 
     #[test]
     fn test_probe_rdp() {
@@ -591,28 +542,29 @@ mod tests {
     fn test_parse_ts_rdp() {
         let buf_1: &[u8] = &[0x03, 0x00, 0x00, 0x25, 0x20, 0xe0, 0x00, 0x00];
         let buf_2: &[u8] = &[
-            0x00, 0x00, 0x00, 0x43, 0x6f, 0x6f, 0x6b, 0x69, 0x65, 0x3a, 0x20,
-            0x6d, 0x73, 0x74, 0x73, 0x68, 0x61, 0x73, 0x68, 0x3d, 0x75, 0x73,
-            0x65, 0x72, 0x31, 0x32, 0x33, 0x0d, 0x0a,
+            0x03, 0x00, 0x00, 0x25, 0x20, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x43, 0x6f, 0x6f,
+            0x6b, 0x69, 0x65, 0x3a, 0x20, 0x6d, 0x73, 0x74, 0x73, 0x68, 0x61, 0x73, 0x68, 0x3d,
+            0x75, 0x73, 0x65, 0x72, 0x31, 0x32, 0x33, 0x0d, 0x0a,
         ];
         let mut state = RdpState::new();
-        assert_eq!(true, state.parse_ts(&buf_1));
+        // will consume 0, request length + 1
+        assert_eq!(AppLayerResult::incomplete(0, 9), state.parse_ts(&buf_1));
         assert_eq!(0, state.transactions.len());
-        assert_eq!(true, state.parse_ts(&buf_2));
+        // exactly aligns with transaction
+        assert_eq!(AppLayerResult::ok(), state.parse_ts(&buf_2));
         assert_eq!(1, state.transactions.len());
-        let item =
-            RdpTransactionItem::X224ConnectionRequest(X224ConnectionRequest {
-                cdt: 0,
-                dst_ref: 0,
-                src_ref: 0,
-                class: 0,
-                options: 0,
-                cookie: Some(RdpCookie {
-                    mstshash: String::from("user123"),
-                }),
-                negotiation_request: None,
-                data: Vec::new(),
-            });
+        let item = RdpTransactionItem::X224ConnectionRequest(X224ConnectionRequest {
+            cdt: 0,
+            dst_ref: 0,
+            src_ref: 0,
+            class: 0,
+            options: 0,
+            cookie: Some(RdpCookie {
+                mstshash: String::from("user123"),
+            }),
+            negotiation_request: None,
+            data: Vec::new(),
+        });
         assert_eq!(item, state.transactions[0].item);
     }
 
@@ -620,20 +572,21 @@ mod tests {
     fn test_parse_ts_other() {
         let buf: &[u8] = &[0x03, 0x00, 0x00, 0x01, 0x00];
         let mut state = RdpState::new();
-        assert_eq!(false, state.parse_ts(&buf));
+        assert_eq!(AppLayerResult::err(), state.parse_ts(&buf));
     }
 
     #[test]
     fn test_parse_tc_rdp() {
         let buf_1: &[u8] = &[0x03, 0x00, 0x00, 0x09, 0x02];
-        let buf_2: &[u8] = &[0xf0, 0x80, 0x7f, 0x66];
+        let buf_2: &[u8] = &[0x03, 0x00, 0x00, 0x09, 0x02, 0xf0, 0x80, 0x7f, 0x66];
         let mut state = RdpState::new();
-        assert_eq!(true, state.parse_tc(&buf_1));
+        // will consume 0, request length + 1
+        assert_eq!(AppLayerResult::incomplete(0, 6), state.parse_tc(&buf_1));
         assert_eq!(0, state.transactions.len());
-        assert_eq!(true, state.parse_tc(&buf_2));
+        // exactly aligns with transaction
+        assert_eq!(AppLayerResult::ok(), state.parse_tc(&buf_2));
         assert_eq!(1, state.transactions.len());
-        let item =
-            RdpTransactionItem::McsConnectResponse(McsConnectResponse {});
+        let item = RdpTransactionItem::McsConnectResponse(McsConnectResponse {});
         assert_eq!(item, state.transactions[0].item);
     }
 
@@ -641,7 +594,7 @@ mod tests {
     fn test_parse_tc_other() {
         let buf: &[u8] = &[0x03, 0x00, 0x00, 0x01, 0x00];
         let mut state = RdpState::new();
-        assert_eq!(false, state.parse_tc(&buf));
+        assert_eq!(AppLayerResult::err(), state.parse_tc(&buf));
     }
 
     #[test]
