@@ -15,7 +15,9 @@
  * 02110-1301, USA.
  */
 
-use nom::{rest, le_u8, le_u16, le_u32};
+use nom::IResult;
+use nom::combinator::rest;
+use nom::number::streaming::{le_u8, le_u16, le_u32};
 
 #[derive(Debug,PartialEq)]
 pub struct NTLMSSPVersion {
@@ -56,6 +58,10 @@ pub struct NTLMSSPAuthRecord<'a> {
     pub version: Option<NTLMSSPVersion>,
 }
 
+fn parse_ntlm_auth_nego_flags(i:&[u8]) -> IResult<&[u8],(u8,u8,u32)> {
+    bits!(i, tuple!(take_bits!(6u8),take_bits!(1u8),take_bits!(25u32)))
+}
+
 named!(pub parse_ntlm_auth_record<NTLMSSPAuthRecord>,
     do_parse!(
             _lm_blob_len: le_u16
@@ -82,7 +88,7 @@ named!(pub parse_ntlm_auth_record<NTLMSSPAuthRecord>,
          >> _ssnkey_blob_maxlen: le_u16
          >> _ssnkey_blob_offset: le_u32
 
-         >> nego_flags: bits!(tuple!(take_bits!(u8, 6),take_bits!(u8,1),take_bits!(u32,25)))
+         >> nego_flags: parse_ntlm_auth_nego_flags
          >> version: cond!(nego_flags.1==1, parse_ntlm_auth_version)
 
          // subtrack 12 as idenfier (8) and type (4) are cut before we are called
@@ -112,7 +118,8 @@ pub struct NTLMSSPRecord<'a> {
 
 named!(pub parse_ntlmssp<NTLMSSPRecord>,
     do_parse!(
-            take_until_and_consume!("NTLMSSP\x00")
+            take_until!("NTLMSSP\x00")
+        >>  tag!("NTLMSSP\x00")
         >>  msg_type: le_u32
         >>  data: rest
         >>  (NTLMSSPRecord {
