@@ -35,9 +35,8 @@ use std::collections::HashMap;
 use nom;
 
 use crate::core::*;
-use crate::log::*;
 use crate::applayer;
-use crate::applayer::LoggerFlags;
+use crate::applayer::{AppLayerResult, AppLayerTxData};
 
 use crate::smb::nbss_records::*;
 use crate::smb::smb1_records::*;
@@ -561,12 +560,9 @@ pub struct SMBTransaction {
     /// Command specific data
     pub type_data: Option<SMBTransactionTypeData>,
 
-    /// detection engine flags for use by detection engine
-    detect_flags_ts: u64,
-    detect_flags_tc: u64,
-    pub logged: LoggerFlags,
     pub de_state: Option<*mut DetectEngineState>,
     pub events: *mut AppLayerDecoderEvents,
+    pub tx_data: AppLayerTxData,
 }
 
 impl SMBTransaction {
@@ -578,11 +574,9 @@ impl SMBTransaction {
             request_done: false,
             response_done: false,
             type_data: None,
-            detect_flags_ts: 0,
-            detect_flags_tc: 0,
-            logged: LoggerFlags::new(),
             de_state: None,
             events: std::ptr::null_mut(),
+            tx_data: AppLayerTxData::new(),
         }
     }
 
@@ -761,10 +755,6 @@ pub struct SMBState<> {
     // requests for DCERPC.
     pub ssnguid2vec_map: HashMap<SMBHashKeyHdrGuid, Vec<u8>>,
 
-    /// TCP segments defragmentation buffer
-    pub tcp_buffer_ts: Vec<u8>,
-    pub tcp_buffer_tc: Vec<u8>,
-
     pub files: SMBFiles,
 
     skip_ts: u32,
@@ -787,6 +777,7 @@ pub struct SMBState<> {
     /// true as long as we have file txs that are in a post-gap
     /// state. It means we'll do extra house keeping for those.
     check_post_gap_file_txs: bool,
+    post_gap_files_checked: bool,
 
     /// transactions list
     pub transactions: Vec<SMBTransaction>,
@@ -817,8 +808,6 @@ impl SMBState {
             ssn2vecoffset_map:HashMap::new(),
             ssn2tree_map:HashMap::new(),
             ssnguid2vec_map:HashMap::new(),
-            tcp_buffer_ts:Vec::new(),
-            tcp_buffer_tc:Vec::new(),
             files: SMBFiles::new(),
             skip_ts:0,
             skip_tc:0,
@@ -833,6 +822,7 @@ impl SMBState {
             ts_trunc: false,
             tc_trunc: false,
             check_post_gap_file_txs: false,
+            post_gap_files_checked: false,
             transactions: Vec::new(),
             tx_id:0,
             dialect:0,
@@ -930,6 +920,13 @@ impl SMBState {
         return None;
     }
 
+    fn update_ts(&mut self, ts: u64) {
+        if ts != self.ts {
+            self.ts = ts;
+            self.post_gap_files_checked = false;
+        }
+    }
+
     /* generic TX has no type_data and is only used to
      * track a single cmd request/reply pair. */
 
@@ -1157,11 +1154,13 @@ impl SMBState {
     {
         let mut post_gap_txs = false;
         for tx in &mut self.transactions {
-            if let Some(SMBTransactionTypeData::FILE(ref f)) = tx.type_data {
+            if let Some(SMBTransactionTypeData::FILE(ref mut f)) = tx.type_data {
                 if f.post_gap_ts > 0 {
                     if self.ts > f.post_gap_ts {
                         tx.request_done = true;
                         tx.response_done = true;
+                        let (files, flags) = self.files.get(f.direction);
+                        f.file_tracker.trunc(files, flags);
                     } else {
                         post_gap_txs = true;
                     }
@@ -1328,6 +1327,7 @@ impl SMBState {
 
                                 }
                             } else if smb.version == 0xfe_u8 { // SMB2
+                                SCLogDebug!("NBSS record {:?}", nbss_part_hdr);
                                 SCLogDebug!("SMBv2 record");
                                 match parse_smb2_request_record(&nbss_part_hdr.data) {
                                     Ok((_, ref smb_record)) => {
@@ -1336,6 +1336,7 @@ impl SMBState {
                                         if smb_record.command == SMB2_COMMAND_WRITE {
                                             smb2_write_request_record(self, smb_record);
                                             let consumed = input.len() - output.len();
+                                            SCLogDebug!("consumed {}", consumed);
                                             return consumed;
                                         }
                                     },
@@ -1355,35 +1356,14 @@ impl SMBState {
     }
 
     /// Parsing function, handling TCP chunks fragmentation
-    pub fn parse_tcp_data_ts<'b>(&mut self, i: &'b[u8]) -> u32
+    pub fn parse_tcp_data_ts<'b>(&mut self, i: &'b[u8]) -> AppLayerResult
     {
-        let mut v : Vec<u8>;
-        //println!("parse_tcp_data_ts ({})",i.len());
-        //println!("{:?}",i);
-        // Check if TCP data is being defragmented
-        let tcp_buffer = match self.tcp_buffer_ts.len() {
-            0 => i,
-            _ => {
-                v = self.tcp_buffer_ts.split_off(0);
-                if self.tcp_buffer_ts.len() + i.len() > 100000 {
-                    self.set_event(SMBEvent::RecordOverflow);
-                    return 1;
-                };
-                v.extend_from_slice(i);
-                v.as_slice()
-            },
-        };
-        //println!("tcp_buffer ({})",tcp_buffer.len());
-        let mut cur_i = tcp_buffer;
-        if cur_i.len() > 1000000 {
-            self.set_event(SMBEvent::RecordOverflow);
-            return 1;
-        }
+        let mut cur_i = i;
         let consumed = self.handle_skip(STREAM_TOSERVER, cur_i.len() as u32);
         if consumed > 0 {
             if consumed > cur_i.len() as u32 {
                 self.set_event(SMBEvent::InternalError);
-                return 1;
+                return AppLayerResult::err();
             }
             cur_i = &cur_i[consumed as usize..];
         }
@@ -1393,30 +1373,42 @@ impl SMBState {
         if consumed > 0 {
             if consumed > cur_i.len() as u32 {
                 self.set_event(SMBEvent::InternalError);
-                return 1;
+                return AppLayerResult::err();
             }
             cur_i = &cur_i[consumed as usize..];
         }
+        if cur_i.len() == 0 {
+            return AppLayerResult::ok();
+        }
         // gap
         if self.ts_gap {
             SCLogDebug!("TS trying to catch up after GAP (input {})", cur_i.len());
-            match search_smb_record(cur_i) {
-                Ok((_, pg)) => {
-                    SCLogDebug!("smb record found");
-                    let smb2_offset = cur_i.len() - pg.len();
-                    if smb2_offset < 4 {
-                        return 0;
-                    }
-                    let nbss_offset = smb2_offset - 4;
-                    cur_i = &cur_i[nbss_offset..];
+            while cur_i.len() > 0 { // min record size
+                match search_smb_record(cur_i) {
+                    Ok((_, pg)) => {
+                        SCLogDebug!("smb record found");
+                        let smb2_offset = cur_i.len() - pg.len();
+                        if smb2_offset < 4 {
+                            cur_i = &cur_i[smb2_offset+4..];
+                            continue; // see if we have another record in our data
+                        }
+                        let nbss_offset = smb2_offset - 4;
+                        cur_i = &cur_i[nbss_offset..];
 
-                    self.ts_gap = false;
-                },
-                _ => {
-                    SCLogDebug!("smb record NOT found");
-                    self.tcp_buffer_ts.extend_from_slice(cur_i);
-                    return 0;
-                },
+                        self.ts_gap = false;
+                        break;
+                    },
+                    _ => {
+                        let mut consumed = i.len();
+                        if consumed < 4 {
+                            consumed = 0;
+                        } else {
+                            consumed = consumed - 3;
+                        }
+                        SCLogDebug!("smb record NOT found");
+                        return AppLayerResult::incomplete(consumed as u32, 8);
+                    },
+                }
             }
         }
         while cur_i.len() > 0 { // min record size
@@ -1436,7 +1428,7 @@ impl SMBState {
                                         },
                                         _ => {
                                             self.set_event(SMBEvent::MalformedData);
-                                            return 1;
+                                            return AppLayerResult::err();
                                         },
                                     }
                                 } else if smb.version == 0xfe_u8 { // SMB2
@@ -1452,7 +1444,7 @@ impl SMBState {
                                             },
                                             _ => {
                                                 self.set_event(SMBEvent::MalformedData);
-                                                return 1;
+                                                return AppLayerResult::err();
                                             },
                                         }
                                     }
@@ -1466,7 +1458,7 @@ impl SMBState {
                                             },
                                             _ => {
                                                 self.set_event(SMBEvent::MalformedData);
-                                                return 1;
+                                                return AppLayerResult::err();
                                             },
                                         }
                                     }
@@ -1474,7 +1466,7 @@ impl SMBState {
                             },
                             _ => {
                                 self.set_event(SMBEvent::MalformedData);
-                                return 1;
+                                return AppLayerResult::err();
                             },
                         }
                     } else {
@@ -1482,25 +1474,44 @@ impl SMBState {
                     }
                     cur_i = rem;
                 },
-                Err(nom::Err::Incomplete(_)) => {
-                    let consumed = self.parse_tcp_data_ts_partial(cur_i);
-                    cur_i = &cur_i[consumed ..];
-
-                    self.tcp_buffer_ts.extend_from_slice(cur_i);
-                    break;
+                Err(nom::Err::Incomplete(needed)) => {
+                    if let nom::Needed::Size(n) = needed {
+                        // 512 is the minimum for parse_tcp_data_ts_partial
+                        if n >= 512 && cur_i.len() < 512 {
+                            let total_consumed = i.len() - cur_i.len();
+                            return AppLayerResult::incomplete(total_consumed as u32, 512);
+                        }
+                        let consumed = self.parse_tcp_data_ts_partial(cur_i);
+                        if consumed == 0 {
+                            // if we consumed none we will buffer the entire record
+                            let total_consumed = i.len() - cur_i.len();
+                            SCLogDebug!("setting consumed {} need {} needed {:?} total input {}",
+                                    total_consumed, n, needed, i.len());
+                            let need = n + 4; // Incomplete returns size of data minus NBSS header
+                            return AppLayerResult::incomplete(total_consumed as u32, need as u32);
+                        }
+                        // tracking a write record, which we don't need to
+                        // queue up at the stream level, but can feed to us
+                        // in small chunks
+                        return AppLayerResult::ok();
+                    } else {
+                        self.set_event(SMBEvent::InternalError);
+                        return AppLayerResult::err();
+                    }
                 },
                 Err(_) => {
                     self.set_event(SMBEvent::MalformedData);
-                    return 1;
+                    return AppLayerResult::err();
                 },
             }
         };
 
         self.post_gap_housekeeping(STREAM_TOSERVER);
-        if self.check_post_gap_file_txs {
+        if self.check_post_gap_file_txs && !self.post_gap_files_checked {
             self.post_gap_housekeeping_for_files();
+            self.post_gap_files_checked = true;
         }
-        0
+        AppLayerResult::ok()
     }
 
     /// return bytes consumed
@@ -1584,33 +1595,14 @@ impl SMBState {
     }
 
     /// Parsing function, handling TCP chunks fragmentation
-    pub fn parse_tcp_data_tc<'b>(&mut self, i: &'b[u8]) -> u32
+    pub fn parse_tcp_data_tc<'b>(&mut self, i: &'b[u8]) -> AppLayerResult
     {
-        let mut v : Vec<u8>;
-        // Check if TCP data is being defragmented
-        let tcp_buffer = match self.tcp_buffer_tc.len() {
-            0 => i,
-            _ => {
-                v = self.tcp_buffer_tc.split_off(0);
-                if self.tcp_buffer_tc.len() + i.len() > 100000 {
-                    self.set_event(SMBEvent::RecordOverflow);
-                    return 1;
-                };
-                v.extend_from_slice(i);
-                v.as_slice()
-            },
-        };
-        let mut cur_i = tcp_buffer;
-        SCLogDebug!("cur_i.len {}", cur_i.len());
-        if cur_i.len() > 100000 {
-            self.set_event(SMBEvent::RecordOverflow);
-            return 1;
-        }
+        let mut cur_i = i;
         let consumed = self.handle_skip(STREAM_TOCLIENT, cur_i.len() as u32);
         if consumed > 0 {
             if consumed > cur_i.len() as u32 {
                 self.set_event(SMBEvent::InternalError);
-                return 1;
+                return AppLayerResult::err();
             }
             cur_i = &cur_i[consumed as usize..];
         }
@@ -1620,30 +1612,42 @@ impl SMBState {
         if consumed > 0 {
             if consumed > cur_i.len() as u32 {
                 self.set_event(SMBEvent::InternalError);
-                return 1;
+                return AppLayerResult::err();
             }
             cur_i = &cur_i[consumed as usize..];
         }
+        if cur_i.len() == 0 {
+            return AppLayerResult::ok();
+        }
         // gap
         if self.tc_gap {
             SCLogDebug!("TC trying to catch up after GAP (input {})", cur_i.len());
-            match search_smb_record(cur_i) {
-                Ok((_, pg)) => {
-                    SCLogDebug!("smb record found");
-                    let smb2_offset = cur_i.len() - pg.len();
-                    if smb2_offset < 4 {
-                        return 0;
-                    }
-                    let nbss_offset = smb2_offset - 4;
-                    cur_i = &cur_i[nbss_offset..];
+            while cur_i.len() > 0 { // min record size
+                match search_smb_record(cur_i) {
+                    Ok((_, pg)) => {
+                        SCLogDebug!("smb record found");
+                        let smb2_offset = cur_i.len() - pg.len();
+                        if smb2_offset < 4 {
+                            cur_i = &cur_i[smb2_offset+4..];
+                            continue; // see if we have another record in our data
+                        }
+                        let nbss_offset = smb2_offset - 4;
+                        cur_i = &cur_i[nbss_offset..];
 
-                    self.tc_gap = false;
-                },
-                _ => {
-                    SCLogDebug!("smb record NOT found");
-                    self.tcp_buffer_tc.extend_from_slice(cur_i);
-                    return 0;
-                },
+                        self.tc_gap = false;
+                        break;
+                    },
+                    _ => {
+                        let mut consumed = i.len();
+                        if consumed < 4 {
+                            consumed = 0;
+                        } else {
+                            consumed = consumed - 3;
+                        }
+                        SCLogDebug!("smb record NOT found");
+                        return AppLayerResult::incomplete(consumed as u32, 8);
+                    },
+                }
             }
         }
         while cur_i.len() > 0 { // min record size
@@ -1663,7 +1667,7 @@ impl SMBState {
                                         },
                                         _ => {
                                             self.set_event(SMBEvent::MalformedData);
-                                            return 1;
+                                            return AppLayerResult::err();
                                         },
                                     }
                                 } else if smb.version == 0xfe_u8 { // SMB2
@@ -1677,7 +1681,7 @@ impl SMBState {
                                             },
                                             _ => {
                                                 self.set_event(SMBEvent::MalformedData);
-                                                return 1;
+                                                return AppLayerResult::err();
                                             },
                                         }
                                     }
@@ -1691,7 +1695,7 @@ impl SMBState {
                                             },
                                             _ => {
                                                 self.set_event(SMBEvent::MalformedData);
-                                                return 1;
+                                                return AppLayerResult::err();
                                             },
                                         }
                                     }
@@ -1703,7 +1707,7 @@ impl SMBState {
                             },
                             Err(_) => {
                                 self.set_event(SMBEvent::MalformedData);
-                                return 1;
+                                return AppLayerResult::err();
                             },
                         }
                     } else {
@@ -1713,33 +1717,48 @@ impl SMBState {
                 },
                 Err(nom::Err::Incomplete(needed)) => {
                     SCLogDebug!("INCOMPLETE have {} needed {:?}", cur_i.len(), needed);
-                    let consumed = self.parse_tcp_data_tc_partial(cur_i);
-                    cur_i = &cur_i[consumed ..];
-
-                    SCLogDebug!("INCOMPLETE have {}", cur_i.len());
-                    self.tcp_buffer_tc.extend_from_slice(cur_i);
-                    break;
+                    if let nom::Needed::Size(n) = needed {
+                        // 512 is the minimum for parse_tcp_data_tc_partial
+                        if n >= 512 && cur_i.len() < 512 {
+                            let total_consumed = i.len() - cur_i.len();
+                            return AppLayerResult::incomplete(total_consumed as u32, 512);
+                        }
+                        let consumed = self.parse_tcp_data_tc_partial(cur_i);
+                        if consumed == 0 {
+                            // if we consumed none we will buffer the entire record
+                            let total_consumed = i.len() - cur_i.len();
+                            SCLogDebug!("setting consumed {} need {} needed {:?} total input {}",
+                                    total_consumed, n, needed, i.len());
+                            let need = n + 4; // Incomplete returns size of data minus NBSS header
+                            return AppLayerResult::incomplete(total_consumed as u32, need as u32);
+                        }
+                        // tracking a read record, which we don't need to
+                        // queue up at the stream level, but can feed to us
+                        // in small chunks
+                        return AppLayerResult::ok();
+                    } else {
+                        self.set_event(SMBEvent::InternalError);
+                        return AppLayerResult::err();
+                    }
                 },
                 Err(_) => {
                     self.set_event(SMBEvent::MalformedData);
-                    return 1;
+                    return AppLayerResult::err();
                 },
             }
         };
         self.post_gap_housekeeping(STREAM_TOCLIENT);
-        if self.check_post_gap_file_txs {
+        if self.check_post_gap_file_txs && !self.post_gap_files_checked {
             self.post_gap_housekeeping_for_files();
+            self.post_gap_files_checked = true;
         }
         self._debug_tx_stats();
-        0
+        AppLayerResult::ok()
     }
 
     /// handle a gap in the TOSERVER direction
     /// returns: 0 ok, 1 unrecoverable error
-    pub fn parse_tcp_data_ts_gap(&mut self, gap_size: u32) -> u32 {
-        if self.tcp_buffer_ts.len() > 0 {
-            self.tcp_buffer_ts.clear();
-        }
+    pub fn parse_tcp_data_ts_gap(&mut self, gap_size: u32) -> AppLayerResult {
         let consumed = self.handle_skip(STREAM_TOSERVER, gap_size);
         if consumed < gap_size {
             let new_gap_size = gap_size - consumed;
@@ -1749,21 +1768,18 @@ impl SMBState {
             if consumed2 > new_gap_size {
                 SCLogDebug!("consumed more than GAP size: {} > {}", consumed2, new_gap_size);
                 self.set_event(SMBEvent::InternalError);
-                return 1;
+                return AppLayerResult::err();
             }
         }
         SCLogDebug!("GAP of size {} in toserver direction", gap_size);
         self.ts_ssn_gap = true;
         self.ts_gap = true;
-        return 0
+        return AppLayerResult::ok();
     }
 
     /// handle a gap in the TOCLIENT direction
     /// returns: 0 ok, 1 unrecoverable error
-    pub fn parse_tcp_data_tc_gap(&mut self, gap_size: u32) -> u32 {
-        if self.tcp_buffer_tc.len() > 0 {
-            self.tcp_buffer_tc.clear();
-        }
+    pub fn parse_tcp_data_tc_gap(&mut self, gap_size: u32) -> AppLayerResult {
         let consumed = self.handle_skip(STREAM_TOCLIENT, gap_size);
         if consumed < gap_size {
             let new_gap_size = gap_size - consumed;
@@ -1773,19 +1789,18 @@ impl SMBState {
             if consumed2 > new_gap_size {
                 SCLogDebug!("consumed more than GAP size: {} > {}", consumed2, new_gap_size);
                 self.set_event(SMBEvent::InternalError);
-                return 1;
+                return AppLayerResult::err();
             }
         }
         SCLogDebug!("GAP of size {} in toclient direction", gap_size);
         self.tc_ssn_gap = true;
         self.tc_gap = true;
-        return 0
+        return AppLayerResult::ok();
     }
 
     pub fn trunc_ts(&mut self) {
         SCLogDebug!("TRUNC TS");
         self.ts_trunc = true;
-        self.tcp_buffer_ts.clear();
 
         for tx in &mut self.transactions {
             if !tx.request_done {
@@ -1797,7 +1812,6 @@ impl SMBState {
     pub fn trunc_tc(&mut self) {
         SCLogDebug!("TRUNC TC");
         self.tc_trunc = true;
-        self.tcp_buffer_tc.clear();
 
         for tx in &mut self.transactions {
             if !tx.response_done {
@@ -1810,7 +1824,7 @@ impl SMBState {
 
 /// Returns *mut SMBState
 #[no_mangle]
-pub extern "C" fn rs_smb_state_new() -> *mut std::os::raw::c_void {
+pub extern "C" fn rs_smb_state_new(_orig_state: *mut std::os::raw::c_void, _orig_proto: AppProto) -> *mut std::os::raw::c_void {
     let state = SMBState::new();
     let boxed = Box::new(state);
     SCLogDebug!("allocating state");
@@ -1836,7 +1850,7 @@ pub extern "C" fn rs_smb_parse_request_tcp(flow: &mut Flow,
                                        input_len: u32,
                                        _data: *mut std::os::raw::c_void,
                                        flags: u8)
-                                       -> i8
+                                       -> AppLayerResult
 {
     let buf = unsafe{std::slice::from_raw_parts(input, input_len as usize)};
     SCLogDebug!("parsing {} bytes of request data", input_len);
@@ -1846,24 +1860,17 @@ pub extern "C" fn rs_smb_parse_request_tcp(flow: &mut Flow,
         state.ts_gap = true;
     }
 
-    state.ts = flow.get_last_time().as_secs();
-    if state.parse_tcp_data_ts(buf) == 0 {
-        return 1;
-    } else {
-        return -1;
-    }
+    state.update_ts(flow.get_last_time().as_secs());
+    state.parse_tcp_data_ts(buf)
 }
 
 #[no_mangle]
 pub extern "C" fn rs_smb_parse_request_tcp_gap(
                                         state: &mut SMBState,
                                         input_len: u32)
-                                        -> i8
+                                        -> AppLayerResult
 {
-    if state.parse_tcp_data_ts_gap(input_len as u32) == 0 {
-        return 1;
-    }
-    return -1;
+    state.parse_tcp_data_ts_gap(input_len as u32)
 }
 
 
@@ -1875,7 +1882,7 @@ pub extern "C" fn rs_smb_parse_response_tcp(flow: &mut Flow,
                                         input_len: u32,
                                         _data: *mut std::os::raw::c_void,
                                         flags: u8)
-                                        -> i8
+                                        -> AppLayerResult
 {
     SCLogDebug!("parsing {} bytes of response data", input_len);
     let buf = unsafe{std::slice::from_raw_parts(input, input_len as usize)};
@@ -1885,24 +1892,17 @@ pub extern "C" fn rs_smb_parse_response_tcp(flow: &mut Flow,
         state.tc_gap = true;
     }
 
-    state.ts = flow.get_last_time().as_secs();
-    if state.parse_tcp_data_tc(buf) == 0 {
-        return 1;
-    } else {
-        return -1;
-    }
+    state.update_ts(flow.get_last_time().as_secs());
+    state.parse_tcp_data_tc(buf)
 }
 
 #[no_mangle]
 pub extern "C" fn rs_smb_parse_response_tcp_gap(
                                         state: &mut SMBState,
                                         input_len: u32)
-                                        -> i8
+                                        -> AppLayerResult
 {
-    if state.parse_tcp_data_tc_gap(input_len as u32) == 0 {
-        return 1;
-    }
-    return -1;
+    state.parse_tcp_data_tc_gap(input_len as u32)
 }
 
 // probing parser
@@ -1977,9 +1977,8 @@ pub extern "C" fn rs_smb_probe_tcp(direction: u8,
             if hdr.is_smb() {
                 SCLogDebug!("smb found");
                 return 1;
-            } else if hdr.is_valid() {
-                SCLogDebug!("nbss found, assume smb");
-                return 1;
+            } else if hdr.needs_more(){
+                return 0;
             }
         },
         _ => { },
@@ -2065,45 +2064,12 @@ pub extern "C" fn rs_smb_tx_get_alstate_progress(tx: &mut SMBTransaction,
 }
 
 #[no_mangle]
-pub extern "C" fn rs_smb_tx_set_logged(_state: &mut SMBState,
-                                       tx: &mut SMBTransaction,
-                                       bits: u32)
+pub extern "C" fn rs_smb_get_tx_data(
+    tx: *mut std::os::raw::c_void)
+    -> *mut AppLayerTxData
 {
-    tx.logged.set(bits);
-}
-
-#[no_mangle]
-pub extern "C" fn rs_smb_tx_get_logged(_state: &mut SMBState,
-                                       tx: &mut SMBTransaction)
-                                       -> u32
-{
-    return tx.logged.get();
-}
-
-#[no_mangle]
-pub extern "C" fn rs_smb_tx_set_detect_flags(
-                                       tx: &mut SMBTransaction,
-                                       direction: u8,
-                                       flags: u64)
-{
-    if (direction & STREAM_TOSERVER) != 0 {
-        tx.detect_flags_ts = flags as u64;
-    } else {
-        tx.detect_flags_tc = flags as u64;
-    }
-}
-
-#[no_mangle]
-pub extern "C" fn rs_smb_tx_get_detect_flags(
-                                       tx: &mut SMBTransaction,
-                                       direction: u8)
-                                       -> u64
-{
-    if (direction & STREAM_TOSERVER) != 0 {
-        return tx.detect_flags_ts as u64;
-    } else {
-        return tx.detect_flags_tc as u64;
-    }
+    let tx = cast_pointer!(tx, SMBTransaction);
+    return &mut tx.tx_data;
 }
 
 #[no_mangle]
@@ -2164,6 +2130,7 @@ pub extern "C" fn rs_smb_state_get_event_info_by_id(event_id: std::os::raw::c_in
             SMBEvent::MalformedNtlmsspResponse => { "malformed_ntlmssp_response\0" },
             SMBEvent::DuplicateNegotiate => { "duplicate_negotiate\0" },
             SMBEvent::NegotiateMalformedDialects => { "netogiate_malformed_dialects\0" },
+            SMBEvent::FileOverlap => { "file_overlap\0" },
         };
         unsafe{
             *event_name = estr.as_ptr() as *const std::os::raw::c_char;
