@@ -43,35 +43,36 @@
  * \brief Functions to decode ERSPAN Type I and II packets
  */
 
-bool g_erspan_typeI_enabled = false;
-
+/*
+ * \brief ERSPAN Type I was configurable in 5.0.x but is no longer configurable.
+ *
+ * Issue a warning if a configuration setting is found.
+ */
 void DecodeERSPANConfig(void)
 {
     int enabled = 0;
     if (ConfGetBool("decoder.erspan.typeI.enabled", &enabled) == 1) {
-        g_erspan_typeI_enabled = (enabled == 1);
+        SCLogWarning(SC_WARN_ERSPAN_CONFIG,
+                     "ERSPAN Type I is no longer configurable and it is always"
+                     " enabled; ignoring configuration setting.");
     }
-    SCLogDebug("ERSPAN Type I decode support %s", g_erspan_typeI_enabled ? "enabled" : "disabled");
 }
 
 /**
  * \brief ERSPAN Type I
  */
 int DecodeERSPANTypeI(ThreadVars *tv, DecodeThreadVars *dtv, Packet *p,
-                      const uint8_t *pkt, uint32_t len, PacketQueue *pq)
+                      const uint8_t *pkt, uint32_t len)
 {
-    if (unlikely(!g_erspan_typeI_enabled))
-        return TM_ECODE_FAILED;
-
     StatsIncr(tv, dtv->counter_erspan);
 
-    return DecodeEthernet(tv, dtv, p, pkt, len, pq);
+    return DecodeEthernet(tv, dtv, p, pkt, len);
 }
 
 /**
  * \brief ERSPAN Type II
  */
-int DecodeERSPANTypeII(ThreadVars *tv, DecodeThreadVars *dtv, Packet *p, const uint8_t *pkt, uint32_t len, PacketQueue *pq)
+int DecodeERSPAN(ThreadVars *tv, DecodeThreadVars *dtv, Packet *p, const uint8_t *pkt, uint32_t len)
 {
     StatsIncr(tv, dtv->counter_erspan);
 
@@ -101,7 +102,7 @@ int DecodeERSPANTypeII(ThreadVars *tv, DecodeThreadVars *dtv, Packet *p, const u
         p->vlan_idx++;
     }
 
-    return DecodeEthernet(tv, dtv, p, pkt + sizeof(ErspanHdr), len - sizeof(ErspanHdr), pq);
+    return DecodeEthernet(tv, dtv, p, pkt + sizeof(ErspanHdr), len - sizeof(ErspanHdr));
 }
 
 /**
