@@ -121,10 +121,12 @@ DNP3Mapping DNP3FunctionNameMap[] = {
     {"authenticate_resp",    131}
 };
 
+#ifdef UNITTESTS
 static void DetectDNP3FuncRegisterTests(void);
 static void DetectDNP3IndRegisterTests(void);
 static void DetectDNP3ObjRegisterTests(void);
 static void DetectDNP3DataRegisterTests(void);
+#endif
 
 /**
  * \brief Utility function to trim leading and trailing whitespace
@@ -195,7 +197,7 @@ static int DetectEngineInspectDNP3(ThreadVars *tv, DetectEngineCtx *de_ctx,
  */
 static int DetectDNP3FuncParseFunctionCode(const char *str, uint8_t *fc)
 {
-    if (ByteExtractStringUint8(fc, 10, strlen(str), str) >= 0) {
+    if (StringParseUint8(fc, 10, strlen(str), str) >= 0) {
         return 1;
     }
 
@@ -287,7 +289,7 @@ static int DetectDNP3IndParse(const char *str, uint16_t *flags)
 {
     *flags = 0;
 
-    if (ByteExtractStringUint16(flags, 0, strlen(str), str) > 0) {
+    if (StringParseUint16(flags, 0, strlen(str), str) > 0) {
         return 1;
     }
 
@@ -362,11 +364,11 @@ static int DetectDNP3ObjParse(const char *str, uint8_t *group, uint8_t *var)
     *sep = '\0';
     varstr = sep + 1;
 
-    if (ByteExtractStringUint8(group, 0, strlen(groupstr), groupstr) < 0) {
+    if (StringParseUint8(group, 0, strlen(groupstr), groupstr) < 0) {
         return 0;
     }
 
-    if (ByteExtractStringUint8(var, 0, strlen(varstr), varstr) < 0) {
+    if (StringParseUint8(var, 0, strlen(varstr), varstr) < 0) {
         return 0;
     }
 
@@ -414,7 +416,7 @@ fail:
     SCReturnInt(0);
 }
 
-static void DetectDNP3Free(void *ptr)
+static void DetectDNP3Free(DetectEngineCtx *de_ctx, void *ptr)
 {
     SCEnter();
     if (ptr != NULL) {
@@ -498,9 +500,10 @@ static void DetectDNP3FuncRegister(void)
     sigmatch_table[DETECT_AL_DNP3FUNC].AppLayerTxMatch = DetectDNP3FuncMatch;
     sigmatch_table[DETECT_AL_DNP3FUNC].Setup         = DetectDNP3FuncSetup;
     sigmatch_table[DETECT_AL_DNP3FUNC].Free          = DetectDNP3Free;
+#ifdef UNITTESTS
     sigmatch_table[DETECT_AL_DNP3FUNC].RegisterTests =
         DetectDNP3FuncRegisterTests;
-
+#endif
     SCReturn;
 }
 
@@ -516,9 +519,10 @@ static void DetectDNP3IndRegister(void)
     sigmatch_table[DETECT_AL_DNP3IND].AppLayerTxMatch = DetectDNP3IndMatch;
     sigmatch_table[DETECT_AL_DNP3IND].Setup         = DetectDNP3IndSetup;
     sigmatch_table[DETECT_AL_DNP3IND].Free          = DetectDNP3Free;
+#ifdef UNITTESTS
     sigmatch_table[DETECT_AL_DNP3IND].RegisterTests =
         DetectDNP3IndRegisterTests;
-
+#endif
     SCReturn;
 }
 
@@ -534,9 +538,10 @@ static void DetectDNP3ObjRegister(void)
     sigmatch_table[DETECT_AL_DNP3OBJ].AppLayerTxMatch = DetectDNP3ObjMatch;
     sigmatch_table[DETECT_AL_DNP3OBJ].Setup         = DetectDNP3ObjSetup;
     sigmatch_table[DETECT_AL_DNP3OBJ].Free          = DetectDNP3Free;
+#ifdef UNITTESTS
     sigmatch_table[DETECT_AL_DNP3OBJ].RegisterTests =
         DetectDNP3ObjRegisterTests;
-
+#endif
     SCReturn;
 }
 
@@ -561,9 +566,10 @@ static void DetectDNP3DataRegister(void)
     sigmatch_table[DETECT_AL_DNP3DATA].desc          = "make the following content options to match on the re-assembled application buffer";
     sigmatch_table[DETECT_AL_DNP3DATA].url           = "/rules/dnp3-keywords.html#dnp3-data";
     sigmatch_table[DETECT_AL_DNP3DATA].Setup         = DetectDNP3DataSetup;
+#ifdef UNITTESTS
     sigmatch_table[DETECT_AL_DNP3DATA].RegisterTests =
         DetectDNP3DataRegisterTests;
-
+#endif
     sigmatch_table[DETECT_AL_DNP3DATA].flags |= SIGMATCH_NOOPT|SIGMATCH_INFO_STICKY_BUFFER;
 
     DetectAppLayerInspectEngineRegister2("dnp3_data",
@@ -1029,39 +1035,30 @@ static int DetectDNP3DataTest02(void)
     PASS;
 }
 
-#endif
-
 static void DetectDNP3FuncRegisterTests(void)
 {
-#ifdef UNITTESTS
     UtRegisterTest("DetectDNP3FuncParseFunctionCodeTest",
                    DetectDNP3FuncParseFunctionCodeTest);
     UtRegisterTest("DetectDNP3FuncTest01", DetectDNP3FuncTest01);
-#endif
 }
 
 static void DetectDNP3IndRegisterTests(void)
 {
-#ifdef UNITTESTS
     UtRegisterTest("DetectDNP3IndTestParseAsInteger",
                    DetectDNP3IndTestParseAsInteger);
     UtRegisterTest("DetectDNP3IndTestParseByName",
                    DetectDNP3IndTestParseByName);
-#endif
 }
 
 static void DetectDNP3ObjRegisterTests(void)
 {
-#ifdef UNITTESTS
     UtRegisterTest("DetectDNP3ObjParseTest", DetectDNP3ObjParseTest);
     UtRegisterTest("DetectDNP3ObjSetupTest", DetectDNP3ObjSetupTest);
-#endif
 }
 
 void DetectDNP3DataRegisterTests(void)
 {
-#ifdef UNITTESTS
     UtRegisterTest("DetectDNP3DataTest01", DetectDNP3DataTest01);
     UtRegisterTest("DetectDNP3DataTest02", DetectDNP3DataTest02);
-#endif
 }
+#endif
