@@ -1,4 +1,4 @@
-/* Copyright (C) 2007-2016 Open Information Security Foundation
+/* Copyright (C) 2007-2020 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -43,17 +43,18 @@
  */
 #define PARSE_REGEX "^\\s*(<|>)?\\s*([0-9]+)\\s*(?:<>\\s*([0-9]+))?\\s*$"
 
-static pcre *parse_regex;
-static pcre_extra *parse_regex_study;
+static DetectParseRegex parse_regex;
 
 static int DetectICodeMatch(DetectEngineThreadCtx *, Packet *,
         const Signature *, const SigMatchCtx *);
 static int DetectICodeSetup(DetectEngineCtx *, Signature *, const char *);
-void DetectICodeRegisterTests(void);
-void DetectICodeFree(void *);
+#ifdef UNITTESTS
+static void DetectICodeRegisterTests(void);
+#endif
+void DetectICodeFree(DetectEngineCtx *, void *);
 
 static int PrefilterSetupICode(DetectEngineCtx *de_ctx, SigGroupHead *sgh);
-static _Bool PrefilterICodeIsPrefilterable(const Signature *s);
+static bool PrefilterICodeIsPrefilterable(const Signature *s);
 
 /**
  * \brief Registration function for icode: keyword
@@ -66,12 +67,13 @@ void DetectICodeRegister (void)
     sigmatch_table[DETECT_ICODE].Match = DetectICodeMatch;
     sigmatch_table[DETECT_ICODE].Setup = DetectICodeSetup;
     sigmatch_table[DETECT_ICODE].Free = DetectICodeFree;
+#ifdef UNITTESTS
     sigmatch_table[DETECT_ICODE].RegisterTests = DetectICodeRegisterTests;
-
+#endif
     sigmatch_table[DETECT_ICODE].SupportsPrefilter = PrefilterICodeIsPrefilterable;
     sigmatch_table[DETECT_ICODE].SetupPrefilter = PrefilterSetupICode;
 
-    DetectSetupParseRegexes(PARSE_REGEX, &parse_regex, &parse_regex_study);
+    DetectSetupParseRegexes(PARSE_REGEX, &parse_regex);
 }
 
 #define DETECT_ICODE_EQ   PREFILTER_U8HASH_MODE_EQ   /**< "equal" operator */
@@ -139,20 +141,20 @@ static int DetectICodeMatch (DetectEngineThreadCtx *det_ctx, Packet *p,
 /**
  * \brief This function is used to parse icode options passed via icode: keyword
  *
+ * \param de_ctx Pointer to the detection engine context
  * \param icodestr Pointer to the user provided icode options
  *
  * \retval icd pointer to DetectICodeData on success
  * \retval NULL on failure
  */
-static DetectICodeData *DetectICodeParse(const char *icodestr)
+static DetectICodeData *DetectICodeParse(DetectEngineCtx *de_ctx, const char *icodestr)
 {
     DetectICodeData *icd = NULL;
     char *args[3] = {NULL, NULL, NULL};
-#define MAX_SUBSTRINGS 30
     int ret = 0, res = 0;
     int ov[MAX_SUBSTRINGS];
 
-    ret = pcre_exec(parse_regex, parse_regex_study, icodestr, strlen(icodestr), 0, 0, ov, MAX_SUBSTRINGS);
+    ret = DetectParsePcreExec(&parse_regex, icodestr, 0, 0, ov, MAX_SUBSTRINGS);
     if (ret < 1 || ret > 4) {
         SCLogError(SC_ERR_PCRE_MATCH, "pcre_exec parse error, ret %" PRId32 ", string %s", ret, icodestr);
         goto error;
@@ -184,7 +186,7 @@ static DetectICodeData *DetectICodeParse(const char *icodestr)
             goto error;
         }
         /* we have only a comparison ("<", ">") */
-        if (ByteExtractStringUint8(&icd->code1, 10, 0, args[1]) < 0) {
+        if (StringParseUint8(&icd->code1, 10, 0, args[1]) < 0) {
             SCLogError(SC_ERR_INVALID_ARGUMENT, "specified icmp code %s is not "
                                         "valid", args[1]);
             goto error;
@@ -195,12 +197,12 @@ static DetectICodeData *DetectICodeParse(const char *icodestr)
         /* we have a range ("<>") */
         if (args[2] != NULL) {
             icd->mode = (uint8_t) DETECT_ICODE_RN;
-            if (ByteExtractStringUint8(&icd->code1, 10, 0, args[1]) < 0) {
+            if (StringParseUint8(&icd->code1, 10, 0, args[1]) < 0) {
                 SCLogError(SC_ERR_INVALID_ARGUMENT, "specified icmp code %s is not "
                                             "valid", args[1]);
                 goto error;
             }
-            if (ByteExtractStringUint8(&icd->code2, 10, 0, args[2]) < 0) {
+            if (StringParseUint8(&icd->code2, 10, 0, args[2]) < 0) {
                 SCLogError(SC_ERR_INVALID_ARGUMENT, "specified icmp code %s is not "
                                             "valid", args[2]);
                 goto error;
@@ -214,7 +216,7 @@ static DetectICodeData *DetectICodeParse(const char *icodestr)
             }
         } else { /* we have an equality */
             icd->mode = DETECT_ICODE_EQ;
-            if (ByteExtractStringUint8(&icd->code1, 10, 0, args[1]) < 0) {
+            if (StringParseUint8(&icd->code1, 10, 0, args[1]) < 0) {
                 SCLogError(SC_ERR_INVALID_ARGUMENT, "specified icmp code %s is not "
                                                     "valid", args[1]);
                 goto error;
@@ -234,7 +236,7 @@ error:
             SCFree(args[i]);
     }
     if (icd != NULL)
-        DetectICodeFree(icd);
+        DetectICodeFree(de_ctx, icd);
     return NULL;
 }
 
@@ -254,7 +256,7 @@ static int DetectICodeSetup(DetectEngineCtx *de_ctx, Signature *s, const char *i
     DetectICodeData *icd = NULL;
     SigMatch *sm = NULL;
 
-    icd = DetectICodeParse(icodestr);
+    icd = DetectICodeParse(NULL, icodestr);
     if (icd == NULL) goto error;
 
     sm = SigMatchAlloc();
@@ -269,7 +271,7 @@ static int DetectICodeSetup(DetectEngineCtx *de_ctx, Signature *s, const char *i
     return 0;
 
 error:
-    if (icd != NULL) DetectICodeFree(icd);
+    if (icd != NULL) DetectICodeFree(de_ctx, icd);
     if (sm != NULL) SCFree(sm);
     return -1;
 }
@@ -279,7 +281,7 @@ error:
  *
  * \param ptr pointer to DetectICodeData
  */
-void DetectICodeFree(void *ptr)
+void DetectICodeFree(DetectEngineCtx *de_ctx, void *ptr)
 {
     DetectICodeData *icd = (DetectICodeData *)ptr;
     SCFree(icd);
@@ -320,7 +322,7 @@ PrefilterPacketICodeSet(PrefilterPacketHeaderValue *v, void *smctx)
     v->u8[2] = a->code2;
 }
 
-static _Bool
+static bool
 PrefilterPacketICodeCompare(PrefilterPacketHeaderValue v, void *smctx)
 {
     const DetectICodeData *a = smctx;
@@ -339,7 +341,7 @@ static int PrefilterSetupICode(DetectEngineCtx *de_ctx, SigGroupHead *sgh)
             PrefilterPacketICodeMatch);
 }
 
-static _Bool PrefilterICodeIsPrefilterable(const Signature *s)
+static bool PrefilterICodeIsPrefilterable(const Signature *s)
 {
     const SigMatch *sm;
     for (sm = s->init_data->smlists[DETECT_SM_LIST_MATCH] ; sm != NULL; sm = sm->next) {
@@ -362,11 +364,11 @@ static int DetectICodeParseTest01(void)
 {
     DetectICodeData *icd = NULL;
     int result = 0;
-    icd = DetectICodeParse("8");
+    icd = DetectICodeParse(NULL, "8");
     if (icd != NULL) {
         if (icd->code1 == 8 && icd->mode == DETECT_ICODE_EQ)
             result = 1;
-        DetectICodeFree(icd);
+        DetectICodeFree(NULL, icd);
     }
     return result;
 }
@@ -379,11 +381,11 @@ static int DetectICodeParseTest02(void)
 {
     DetectICodeData *icd = NULL;
     int result = 0;
-    icd = DetectICodeParse(">8");
+    icd = DetectICodeParse(NULL, ">8");
     if (icd != NULL) {
         if (icd->code1 == 8 && icd->mode == DETECT_ICODE_GT)
             result = 1;
-        DetectICodeFree(icd);
+        DetectICodeFree(NULL, icd);
     }
     return result;
 }
@@ -396,11 +398,11 @@ static int DetectICodeParseTest03(void)
 {
     DetectICodeData *icd = NULL;
     int result = 0;
-    icd = DetectICodeParse("<8");
+    icd = DetectICodeParse(NULL, "<8");
     if (icd != NULL) {
         if (icd->code1 == 8 && icd->mode == DETECT_ICODE_LT)
             result = 1;
-        DetectICodeFree(icd);
+        DetectICodeFree(NULL, icd);
     }
     return result;
 }
@@ -413,11 +415,11 @@ static int DetectICodeParseTest04(void)
 {
     DetectICodeData *icd = NULL;
     int result = 0;
-    icd = DetectICodeParse("8<>20");
+    icd = DetectICodeParse(NULL, "8<>20");
     if (icd != NULL) {
         if (icd->code1 == 8 && icd->code2 == 20 && icd->mode == DETECT_ICODE_RN)
             result = 1;
-        DetectICodeFree(icd);
+        DetectICodeFree(NULL, icd);
     }
     return result;
 }
@@ -430,11 +432,11 @@ static int DetectICodeParseTest05(void)
 {
     DetectICodeData *icd = NULL;
     int result = 0;
-    icd = DetectICodeParse("  8 ");
+    icd = DetectICodeParse(NULL, "  8 ");
     if (icd != NULL) {
         if (icd->code1 == 8 && icd->mode == DETECT_ICODE_EQ)
             result = 1;
-        DetectICodeFree(icd);
+        DetectICodeFree(NULL, icd);
     }
     return result;
 }
@@ -447,11 +449,11 @@ static int DetectICodeParseTest06(void)
 {
     DetectICodeData *icd = NULL;
     int result = 0;
-    icd = DetectICodeParse("  >  8 ");
+    icd = DetectICodeParse(NULL, "  >  8 ");
     if (icd != NULL) {
         if (icd->code1 == 8 && icd->mode == DETECT_ICODE_GT)
             result = 1;
-        DetectICodeFree(icd);
+        DetectICodeFree(NULL, icd);
     }
     return result;
 }
@@ -464,11 +466,11 @@ static int DetectICodeParseTest07(void)
 {
     DetectICodeData *icd = NULL;
     int result = 0;
-    icd = DetectICodeParse("  8  <>  20 ");
+    icd = DetectICodeParse(NULL, "  8  <>  20 ");
     if (icd != NULL) {
         if (icd->code1 == 8 && icd->code2 == 20 && icd->mode == DETECT_ICODE_RN)
             result = 1;
-        DetectICodeFree(icd);
+        DetectICodeFree(NULL, icd);
     }
     return result;
 }
@@ -479,10 +481,10 @@ static int DetectICodeParseTest07(void)
 static int DetectICodeParseTest08(void)
 {
     DetectICodeData *icd = NULL;
-    icd = DetectICodeParse("> 8 <> 20");
+    icd = DetectICodeParse(NULL, "> 8 <> 20");
     if (icd == NULL)
         return 1;
-    DetectICodeFree(icd);
+    DetectICodeFree(NULL, icd);
     return 0;
 }
 
@@ -573,14 +575,11 @@ end:
     return result;
 }
 
-#endif /* UNITTESTS */
-
 /**
  * \brief this function registers unit tests for DetectICode
  */
 void DetectICodeRegisterTests(void)
 {
-#ifdef UNITTESTS
     UtRegisterTest("DetectICodeParseTest01", DetectICodeParseTest01);
     UtRegisterTest("DetectICodeParseTest02", DetectICodeParseTest02);
     UtRegisterTest("DetectICodeParseTest03", DetectICodeParseTest03);
@@ -590,5 +589,5 @@ void DetectICodeRegisterTests(void)
     UtRegisterTest("DetectICodeParseTest07", DetectICodeParseTest07);
     UtRegisterTest("DetectICodeParseTest08", DetectICodeParseTest08);
     UtRegisterTest("DetectICodeMatchTest01", DetectICodeMatchTest01);
-#endif /* UNITTESTS */
 }
+#endif /* UNITTESTS */
\ No newline at end of file
