@@ -64,7 +64,7 @@ static InspectionBuffer *GetData(DetectEngineThreadCtx *det_ctx,
         void *txv, const int list_id);
 static void DetectTlsFingerprintSetupCallback(const DetectEngineCtx *de_ctx,
         Signature *s);
-static _Bool DetectTlsFingerprintValidateCallback(const Signature *s,
+static bool DetectTlsFingerprintValidateCallback(const Signature *s,
         const char **sigerror);
 static int g_tls_cert_fingerprint_buffer_id = 0;
 
@@ -148,7 +148,7 @@ static InspectionBuffer *GetData(DetectEngineThreadCtx *det_ctx,
     return buffer;
 }
 
-static _Bool DetectTlsFingerprintValidateCallback(const Signature *s,
+static bool DetectTlsFingerprintValidateCallback(const Signature *s,
                                                   const char **sigerror)
 {
     const SigMatch *sm = s->init_data->smlists[g_tls_cert_fingerprint_buffer_id];
@@ -166,7 +166,7 @@ static _Bool DetectTlsFingerprintValidateCallback(const Signature *s,
             return FALSE;
         }
 
-        _Bool have_delimiters = FALSE;
+        bool have_delimiters = FALSE;
         uint32_t u;
         for (u = 0; u < cd->content_len; u++)
         {
@@ -206,7 +206,7 @@ static void DetectTlsFingerprintSetupCallback(const DetectEngineCtx *de_ctx,
 
         DetectContentData *cd = (DetectContentData *)sm->ctx;
 
-        _Bool changed = FALSE;
+        bool changed = FALSE;
         uint32_t u;
         for (u = 0; u < cd->content_len; u++)
         {
