@@ -29,6 +29,7 @@
 #include "util-atomic.h"
 #include "util-device.h"
 #include "detect-tag.h"
+#include "util-macset.h"
 #include "util-optimize.h"
 
 /* Part of the flow structure, so we declare it here.
@@ -109,6 +110,8 @@ typedef struct AppLayerParserState_ AppLayerParserState;
 
 /* File flags */
 
+#define FLOWFILE_INIT                   0
+
 /** no magic on files in this flow */
 #define FLOWFILE_NO_MAGIC_TS            BIT_U16(0)
 #define FLOWFILE_NO_MAGIC_TC            BIT_U16(1)
@@ -132,6 +135,19 @@ typedef struct AppLayerParserState_ AppLayerParserState;
 #define FLOWFILE_NO_SIZE_TS             BIT_U16(10)
 #define FLOWFILE_NO_SIZE_TC             BIT_U16(11)
 
+#define FLOWFILE_NONE_TS (FLOWFILE_NO_MAGIC_TS | \
+                          FLOWFILE_NO_STORE_TS | \
+                          FLOWFILE_NO_MD5_TS   | \
+                          FLOWFILE_NO_SHA1_TS  | \
+                          FLOWFILE_NO_SHA256_TS| \
+                          FLOWFILE_NO_SIZE_TS)
+#define FLOWFILE_NONE_TC (FLOWFILE_NO_MAGIC_TC | \
+                          FLOWFILE_NO_STORE_TC | \
+                          FLOWFILE_NO_MD5_TC   | \
+                          FLOWFILE_NO_SHA1_TC  | \
+                          FLOWFILE_NO_SHA256_TC| \
+                          FLOWFILE_NO_SIZE_TC)
+#define FLOWFILE_NONE    (FLOWFILE_NONE_TS|FLOWFILE_NONE_TC)
 
 #define FLOW_IS_IPV4(f) \
     (((f)->flags & FLOW_IPV4) == FLOW_IPV4)
@@ -207,6 +223,9 @@ typedef struct AppLayerParserState_ AppLayerParserState;
 #define FLOW_PKT_TOCLIENT_IPONLY_SET    0x10
 #define FLOW_PKT_TOSERVER_FIRST         0x20
 #define FLOW_PKT_TOCLIENT_FIRST         0x40
+/** last pseudo packet in the flow. Can be used to trigger final clean,
+ *  logging, etc. */
+#define FLOW_PKT_LAST_PSEUDO            0x80
 
 #define FLOW_END_FLAG_STATE_NEW         0x01
 #define FLOW_END_FLAG_STATE_ESTABLISHED 0x02
@@ -346,8 +365,33 @@ typedef struct Flow_
     uint8_t proto;
     uint8_t recursion_level;
     uint16_t vlan_id[2];
+    /** how many references exist to this flow *right now*
+     *
+     *  On receiving a packet the counter is incremented while the flow
+     *  bucked is locked, which is also the case on timeout pruning.
+     */
+    FlowRefCount use_cnt;
+
     uint8_t vlan_idx;
 
+    /* track toserver/toclient flow timeout needs */
+    union {
+        struct {
+            uint8_t ffr_ts:4;
+            uint8_t ffr_tc:4;
+        };
+        uint8_t ffr;
+    };
+
+    /** timestamp in seconds of the moment this flow will timeout
+     *  according to the timeout policy. Does *not* take emergency
+     *  mode into account. */
+    uint32_t timeout_at;
+
+    /** Thread ID for the stream/detect portion of this flow */
+    FlowThreadId thread_id[2];
+
+    struct Flow_ *next; /* (hash) list next */
     /** Incoming interface */
     struct LiveDevice_ *livedev;
 
@@ -361,15 +405,11 @@ typedef struct Flow_
 
     /* end of flow "header" */
 
-    SC_ATOMIC_DECLARE(FlowStateType, flow_state);
+    /** timeout policy value in seconds to add to the lastts.tv_sec
+     *  when a packet has been received. */
+    uint32_t timeout_policy;
 
-    /** how many pkts and stream msgs are using the flow *right now*. This
-     *  variable is atomic so not protected by the Flow mutex "m".
-     *
-     *  On receiving a packet the counter is incremented while the flow
-     *  bucked is locked, which is also the case on timeout pruning.
-     */
-    SC_ATOMIC_DECLARE(FlowRefCount, use_cnt);
+    FlowStateType flow_state;
 
     /** flow tenant id, used to setup flow timeout and stream pseudo
      *  packets with the correct tenant id set */
@@ -381,7 +421,6 @@ typedef struct Flow_
     uint32_t flags;         /**< generic flags */
 
     uint16_t file_flags;    /**< file tracking/extraction flags */
-    /* coccinelle: Flow:file_flags:FLOWFILE_ */
 
     /** destination port to be used in protocol detection. This is meant
      *  for use with STARTTLS and HTTP CONNECT detection */
@@ -424,9 +463,6 @@ typedef struct Flow_
      *  stored sgh ptrs are reset. */
     uint32_t de_ctx_version;
 
-    /** Thread ID for the stream/detect portion of this flow */
-    FlowThreadId thread_id[2];
-
     /** ttl tracking */
     uint8_t min_ttl_toserver;
     uint8_t max_ttl_toserver;
@@ -449,14 +485,8 @@ typedef struct Flow_
     /* pointer to the var list */
     GenericVar *flowvar;
 
-    /** hash list pointers, protected by fb->s */
-    struct Flow_ *hnext; /* hash list */
-    struct Flow_ *hprev;
     struct FlowBucket_ *fb;
 
-    /** queue list pointers, protected by queue mutex */
-    struct Flow_ *lnext; /* list */
-    struct Flow_ *lprev;
     struct timeval startts;
 
     uint32_t todstpktcnt;
@@ -496,12 +526,23 @@ typedef struct FlowBypassInfo_ {
     uint64_t todstbytecnt;
 } FlowBypassInfo;
 
+#include "flow-queue.h"
+
+typedef struct FlowLookupStruct_ // TODO name
+{
+    /** thread store of spare queues */
+    FlowQueuePrivate spare_queue;
+    DecodeThreadVars *dtv;
+    FlowQueuePrivate work_queue;
+    uint32_t emerg_spare_sync_stamp;
+} FlowLookupStruct;
+
 /** \brief prepare packet for a life with flow
  *  Set PKT_WANTS_FLOW flag to incidate workers should do a flow lookup
  *  and calc the hash value to be used in the lookup and autofp flow
  *  balancing. */
 void FlowSetupPacket(Packet *p);
-void FlowHandlePacket (ThreadVars *, DecodeThreadVars *, Packet *);
+void FlowHandlePacket (ThreadVars *, FlowLookupStruct *, Packet *);
 void FlowInitConfig (char);
 void FlowPrintQueueInfo (void);
 void FlowShutdown(void);
@@ -519,8 +560,6 @@ int FlowSetProtoEmergencyTimeout(uint8_t ,uint32_t ,uint32_t ,uint32_t);
 int FlowSetProtoFreeFunc (uint8_t , void (*Free)(void *));
 void FlowUpdateQueue(Flow *);
 
-struct FlowQueue_;
-
 int FlowUpdateSpareFlows(void);
 
 static inline void FlowSetNoPacketInspectionFlag(Flow *);
@@ -581,7 +620,7 @@ static inline void FlowIncrUsecnt(Flow *f)
     if (f == NULL)
         return;
 
-    (void) SC_ATOMIC_ADD(f->use_cnt, 1);
+    f->use_cnt++;
 }
 
 /**
@@ -594,7 +633,7 @@ static inline void FlowDecrUsecnt(Flow *f)
     if (f == NULL)
         return;
 
-    (void) SC_ATOMIC_SUB(f->use_cnt, 1);
+    f->use_cnt--;
 }
 
 /** \brief Reference the flow, bumping the flows use_cnt
@@ -638,12 +677,29 @@ static inline int64_t FlowGetId(const Flow *f)
     return id;
 }
 
+static inline void FlowSetEndFlags(Flow *f)
+{
+    const int state = f->flow_state;
+    if (state == FLOW_STATE_NEW)
+        f->flow_end_flags |= FLOW_END_FLAG_STATE_NEW;
+    else if (state == FLOW_STATE_ESTABLISHED)
+        f->flow_end_flags |= FLOW_END_FLAG_STATE_ESTABLISHED;
+    else if (state == FLOW_STATE_CLOSED)
+        f->flow_end_flags |= FLOW_END_FLAG_STATE_CLOSED;
+    else if (state == FLOW_STATE_LOCAL_BYPASSED)
+        f->flow_end_flags |= FLOW_END_FLAG_STATE_BYPASSED;
+#ifdef CAPTURE_OFFLOAD
+    else if (state == FLOW_STATE_CAPTURE_BYPASSED)
+        f->flow_end_flags = FLOW_END_FLAG_STATE_BYPASSED;
+#endif
+}
+
 int FlowClearMemory(Flow *,uint8_t );
 
 AppProto FlowGetAppProtocol(const Flow *f);
 void *FlowGetAppState(const Flow *f);
 uint8_t FlowGetDisruptionFlags(const Flow *f, uint8_t flags);
 
-void FlowHandlePacketUpdate(Flow *f, Packet *p);
+void FlowHandlePacketUpdate(Flow *f, Packet *p, ThreadVars *tv, DecodeThreadVars *dtv);
 
 #endif /* __FLOW_H__ */
