@@ -26,9 +26,7 @@
 
 void JsonDnsLogRegister(void);
 
-#include "app-layer-dns-common.h"
-
-json_t *JsonDNSLogQuery(void *txptr, uint64_t tx_id) __attribute__((nonnull));
-json_t *JsonDNSLogAnswer(void *txptr, uint64_t tx_id) __attribute__((nonnull));
+JsonBuilder *JsonDNSLogQuery(void *txptr, uint64_t tx_id) __attribute__((nonnull));
+JsonBuilder *JsonDNSLogAnswer(void *txptr, uint64_t tx_id) __attribute__((nonnull));
 
 #endif /* __OUTPUT_JSON_DNS_H__ */
