@@ -1,4 +1,4 @@
-/* Copyright (C) 2007-2018 Open Information Security Foundation
+/* Copyright (C) 2007-2020 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -60,12 +60,16 @@
 #include "util-device.h"
 #include "util-validate.h"
 #include "util-crypt.h"
+#include "util-plugin.h"
 
 #include "flow-var.h"
 #include "flow-bit.h"
+#include "flow-storage.h"
 
 #include "source-pcap-file.h"
 
+#include "suricata-plugin.h"
+
 #define DEFAULT_LOG_FILENAME "eve.json"
 #define DEFAULT_ALERT_SYSLOG_FACILITY_STR       "local0"
 #define DEFAULT_ALERT_SYSLOG_FACILITY           LOG_LOCAL0
@@ -75,13 +79,15 @@
 #define MAX_JSON_SIZE 2048
 
 static void OutputJsonDeInitCtx(OutputCtx *);
-static void CreateJSONCommunityFlowId(json_t *js, const Flow *f, const uint16_t seed);
+static void CreateEveCommunityFlowId(JsonBuilder *js, const Flow *f, const uint16_t seed);
 
 static const char *TRAFFIC_ID_PREFIX = "traffic/id/";
 static const char *TRAFFIC_LABEL_PREFIX = "traffic/label/";
 static size_t traffic_id_prefix_len = 0;
 static size_t traffic_label_prefix_len = 0;
 
+const JsonAddrInfo json_addr_info_zero;
+
 void OutputJsonRegister (void)
 {
     OutputRegisterModule(MODULE_NAME, "eve-log", OutputJsonInitCtx);
@@ -132,40 +138,97 @@ json_t *SCJsonString(const char *val)
 /* Default Sensor ID value */
 static int64_t sensor_id = -1; /* -1 = not defined */
 
-/**
- * \brief Create a JSON string from a character sequence
- *
- * \param Pointer to character sequence
- * \param Number of characters to use from the sequence
- * \retval JSON object for the character sequence
- */
-json_t *JsonAddStringN(const char *string, size_t size)
+void EveFileInfo(JsonBuilder *jb, const File *ff, const bool stored)
 {
-    char tmpbuf[size + 1];
+    jb_set_string_from_bytes(jb, "filename", ff->name, ff->name_len);
 
-    memcpy(tmpbuf, string, size);
-    tmpbuf[size] = '\0';
+    jb_open_array(jb, "sid");
+    for (uint32_t i = 0; ff->sid != NULL && i < ff->sid_cnt; i++) {
+        jb_append_uint(jb, ff->sid[i]);
+    }
+    jb_close(jb);
 
-    return SCJsonString(tmpbuf);
+#ifdef HAVE_MAGIC
+    if (ff->magic)
+        jb_set_string(jb, "magic", (char *)ff->magic);
+#endif
+    jb_set_bool(jb, "gaps", ff->flags & FILE_HAS_GAPS);
+    switch (ff->state) {
+        case FILE_STATE_CLOSED:
+            JB_SET_STRING(jb, "state", "CLOSED");
+#ifdef HAVE_NSS
+            if (ff->flags & FILE_MD5) {
+                size_t x;
+                int i;
+                char str[256];
+                for (i = 0, x = 0; x < sizeof(ff->md5); x++) {
+                    i += snprintf(&str[i], 255-i, "%02x", ff->md5[x]);
+                }
+                jb_set_string(jb, "md5", str);
+            }
+            if (ff->flags & FILE_SHA1) {
+                size_t x;
+                int i;
+                char str[256];
+                for (i = 0, x = 0; x < sizeof(ff->sha1); x++) {
+                    i += snprintf(&str[i], 255-i, "%02x", ff->sha1[x]);
+                }
+                jb_set_string(jb, "sha1", str);
+            }
+#endif
+            break;
+        case FILE_STATE_TRUNCATED:
+            JB_SET_STRING(jb, "state", "TRUNCATED");
+            break;
+        case FILE_STATE_ERROR:
+            JB_SET_STRING(jb, "state", "ERROR");
+            break;
+        default:
+            JB_SET_STRING(jb, "state", "UNKNOWN");
+            break;
+    }
+
+#ifdef HAVE_NSS
+    if (ff->flags & FILE_SHA256) {
+        size_t x;
+        int i;
+        char str[256];
+        for (i = 0, x = 0; x < sizeof(ff->sha256); x++) {
+            i += snprintf(&str[i], 255-i, "%02x", ff->sha256[x]);
+        }
+        jb_set_string(jb, "sha256", str);
+    }
+#endif
+
+    if (stored) {
+        JB_SET_TRUE(jb, "stored");
+        jb_set_uint(jb, "file_id", ff->file_store_id);
+    } else {
+        JB_SET_FALSE(jb, "stored");
+    }
+
+    jb_set_uint(jb, "size", FileTrackedSize(ff));
+    if (ff->end > 0) {
+        jb_set_uint(jb, "start", ff->start);
+        jb_set_uint(jb, "end", ff->end);
+    }
+    jb_set_uint(jb, "tx_id", ff->txid);
 }
-static void JsonAddPacketvars(const Packet *p, json_t *js_vars)
+
+static void EveAddPacketVars(const Packet *p, JsonBuilder *js_vars)
 {
     if (p == NULL || p->pktvar == NULL) {
         return;
     }
-    json_t *js_pktvars = NULL;
     PktVar *pv = p->pktvar;
+    bool open = false;
     while (pv != NULL) {
         if (pv->key || pv->id > 0) {
-            if (js_pktvars == NULL) {
-                js_pktvars = json_array();
-                if (js_pktvars == NULL)
-                    break;
-            }
-            json_t *js_pair = json_object();
-            if (js_pair == NULL) {
-                break;
+            if (!open) {
+                jb_open_array(js_vars, "pktvars");
+                open = true;
             }
+            jb_start_object(js_vars);
 
             if (pv->key != NULL) {
                 uint32_t offset = 0;
@@ -179,8 +242,7 @@ static void JsonAddPacketvars(const Packet *p, json_t *js_vars)
                 PrintStringsToBuffer(printable_buf, &offset,
                         sizeof(printable_buf),
                         pv->value, pv->value_len);
-                json_object_set_new(js_pair, (char *)keybuf,
-                        json_string((char *)printable_buf));
+                jb_set_string(js_vars, (char *)keybuf, (char *)printable_buf);
             } else {
                 const char *varname = VarNameStoreLookupById(pv->id, VAR_TYPE_PKT_VAR);
                 uint32_t len = pv->value_len;
@@ -189,16 +251,14 @@ static void JsonAddPacketvars(const Packet *p, json_t *js_vars)
                 PrintStringsToBuffer(printable_buf, &offset,
                         sizeof(printable_buf),
                         pv->value, pv->value_len);
-
-                json_object_set_new(js_pair, varname,
-                        json_string((char *)printable_buf));
+                jb_set_string(js_vars, varname, (char *)printable_buf);
             }
-            json_array_append_new(js_pktvars, js_pair);
+            jb_close(js_vars);
         }
         pv = pv->next;
     }
-    if (js_pktvars) {
-        json_object_set_new(js_vars, "pktvars", js_pktvars);
+    if (open) {
+        jb_close(js_vars);
     }
 }
 
@@ -218,22 +278,16 @@ static bool SCStringHasPrefix(const char *s, const char *prefix)
     return false;
 }
 
-/**
- * \brief Add flow variables to a json object.
- *
- * Adds "flowvars" (map), "flowints" (map) and "flowbits" (array) to
- * the json object provided as js_root.
- */
-static void JsonAddFlowVars(const Flow *f, json_t *js_root, json_t **js_traffic)
+static void EveAddFlowVars(const Flow *f, JsonBuilder *js_root, JsonBuilder **js_traffic)
 {
     if (f == NULL || f->flowvar == NULL) {
         return;
     }
-    json_t *js_flowvars = NULL;
-    json_t *js_traffic_id = NULL;
-    json_t *js_traffic_label = NULL;
-    json_t *js_flowints = NULL;
-    json_t *js_flowbits = NULL;
+    JsonBuilder *js_flowvars = NULL;
+    JsonBuilder *js_traffic_id = NULL;
+    JsonBuilder *js_traffic_label = NULL;
+    JsonBuilder *js_flowints = NULL;
+    JsonBuilder *js_flowbits = NULL;
     GenericVar *gv = f->flowvar;
     while (gv != NULL) {
         if (gv->type == DETECT_FLOWVAR || gv->type == DETECT_FLOWINT) {
@@ -243,7 +297,7 @@ static void JsonAddFlowVars(const Flow *f, json_t *js_root, json_t **js_traffic)
                         VAR_TYPE_FLOW_VAR);
                 if (varname) {
                     if (js_flowvars == NULL) {
-                        js_flowvars = json_array();
+                        js_flowvars = jb_new_array();
                         if (js_flowvars == NULL)
                             break;
                     }
@@ -255,17 +309,13 @@ static void JsonAddFlowVars(const Flow *f, json_t *js_root, json_t **js_traffic)
                             sizeof(printable_buf),
                             fv->data.fv_str.value, fv->data.fv_str.value_len);
 
-                    json_t *js_flowvar = json_object();
-                    if (unlikely(js_flowvar == NULL)) {
-                        break;
-                    }
-                    json_object_set_new(js_flowvar, varname,
-                            json_string((char *)printable_buf));
-                    json_array_append_new(js_flowvars, js_flowvar);
+                    jb_start_object(js_flowvars);
+                    jb_set_string(js_flowvars, varname, (char *)printable_buf);
+                    jb_close(js_flowvars);
                 }
             } else if (fv->datatype == FLOWVAR_TYPE_STR && fv->key != NULL) {
                 if (js_flowvars == NULL) {
-                    js_flowvars = json_array();
+                    js_flowvars = jb_new_array();
                     if (js_flowvars == NULL)
                         break;
                 }
@@ -283,25 +333,19 @@ static void JsonAddFlowVars(const Flow *f, json_t *js_root, json_t **js_traffic)
                         sizeof(printable_buf),
                         fv->data.fv_str.value, fv->data.fv_str.value_len);
 
-                json_t *js_flowvar = json_object();
-                if (unlikely(js_flowvar == NULL)) {
-                    break;
-                }
-                json_object_set_new(js_flowvar, (const char *)keybuf,
-                        json_string((char *)printable_buf));
-                json_array_append_new(js_flowvars, js_flowvar);
+                jb_start_object(js_flowvars);
+                jb_set_string(js_flowvars, (const char *)keybuf, (char *)printable_buf);
+                jb_close(js_flowvars);
             } else if (fv->datatype == FLOWVAR_TYPE_INT) {
                 const char *varname = VarNameStoreLookupById(fv->idx,
                         VAR_TYPE_FLOW_INT);
                 if (varname) {
                     if (js_flowints == NULL) {
-                        js_flowints = json_object();
+                        js_flowints = jb_new_object();
                         if (js_flowints == NULL)
                             break;
                     }
-
-                    json_object_set_new(js_flowints, varname,
-                            json_integer(fv->data.fv_int.value));
+                    jb_set_uint(js_flowints, varname, fv->data.fv_int.value);
                 }
 
             }
@@ -312,89 +356,102 @@ static void JsonAddFlowVars(const Flow *f, json_t *js_root, json_t **js_traffic)
             if (varname) {
                 if (SCStringHasPrefix(varname, TRAFFIC_ID_PREFIX)) {
                     if (js_traffic_id == NULL) {
-                        js_traffic_id = json_array();
+                        js_traffic_id = jb_new_array();
                         if (unlikely(js_traffic_id == NULL)) {
                             break;
                         }
                     }
-                    json_array_append_new(js_traffic_id,
-                            json_string(&varname[traffic_id_prefix_len]));
+                    jb_append_string(js_traffic_id, &varname[traffic_id_prefix_len]);
                 } else if (SCStringHasPrefix(varname, TRAFFIC_LABEL_PREFIX)) {
                     if (js_traffic_label == NULL) {
-                        js_traffic_label = json_array();
+                        js_traffic_label = jb_new_array();
                         if (unlikely(js_traffic_label == NULL)) {
                             break;
                         }
                     }
-                    json_array_append_new(js_traffic_label,
-                            json_string(&varname[traffic_label_prefix_len]));
+                    jb_append_string(js_traffic_label, &varname[traffic_label_prefix_len]);
                 } else {
                     if (js_flowbits == NULL) {
-                        js_flowbits = json_array();
+                        js_flowbits = jb_new_array();
                         if (unlikely(js_flowbits == NULL))
                             break;
                     }
-                    json_array_append_new(js_flowbits, json_string(varname));
+                    jb_append_string(js_flowbits, varname);
                 }
             }
         }
         gv = gv->next;
     }
     if (js_flowbits) {
-        json_object_set_new(js_root, "flowbits", js_flowbits);
+        jb_close(js_flowbits);
+        jb_set_object(js_root, "flowbits", js_flowbits);
+        jb_free(js_flowbits);
     }
     if (js_flowints) {
-        json_object_set_new(js_root, "flowints", js_flowints);
+        jb_close(js_flowints);
+        jb_set_object(js_root, "flowints", js_flowints);
+        jb_free(js_flowints);
     }
     if (js_flowvars) {
-        json_object_set_new(js_root, "flowvars", js_flowvars);
+        jb_close(js_flowvars);
+        jb_set_object(js_root, "flowvars", js_flowvars);
+        jb_free(js_flowvars);
     }
 
     if (js_traffic_id != NULL || js_traffic_label != NULL) {
-        *js_traffic = json_object();
+        *js_traffic = jb_new_object();
         if (likely(*js_traffic != NULL)) {
             if (js_traffic_id != NULL) {
-                json_object_set_new(*js_traffic, "id", js_traffic_id);
+                jb_close(js_traffic_id);
+                jb_set_object(*js_traffic, "id", js_traffic_id);
+                jb_free(js_traffic_id);
             }
             if (js_traffic_label != NULL) {
-                json_object_set_new(*js_traffic, "label", js_traffic_label);
+                jb_close(js_traffic_label);
+                jb_set_object(*js_traffic, "label", js_traffic_label);
+                jb_free(js_traffic_label);
             }
+            jb_close(*js_traffic);
         }
     }
 }
 
-/**
- * \brief Add top-level metadata to the eve json object.
- */
-static void JsonAddMetadata(const Packet *p, const Flow *f, json_t *js)
+static void EveAddMetadata(const Packet *p, const Flow *f, JsonBuilder *js)
 {
     if ((p && p->pktvar) || (f && f->flowvar)) {
-        json_t *js_vars = json_object();
-        json_t *js_traffic = NULL;
+        JsonBuilder *js_vars = jb_new_object();
         if (js_vars) {
             if (f && f->flowvar) {
-                JsonAddFlowVars(f, js_vars, &js_traffic);
+                JsonBuilder *js_traffic = NULL;
+                EveAddFlowVars(f, js_vars, &js_traffic);
                 if (js_traffic != NULL) {
-                    json_object_set_new(js, "traffic", js_traffic);
+                    jb_set_object(js, "traffic", js_traffic);
+                    jb_free(js_traffic);
                 }
             }
             if (p && p->pktvar) {
-                JsonAddPacketvars(p, js_vars);
+                EveAddPacketVars(p, js_vars);
             }
-
-            json_object_set_new(js, "metadata", js_vars);
+            jb_close(js_vars);
+            jb_set_object(js, "metadata", js_vars);
+            jb_free(js_vars);
         }
     }
 }
 
-void JsonAddCommonOptions(const OutputJsonCommonSettings *cfg,
-        const Packet *p, const Flow *f, json_t *js)
+int CreateJSONEther(JsonBuilder *parent, const Packet *p, const Flow *f);
+
+void EveAddCommonOptions(const OutputJsonCommonSettings *cfg,
+        const Packet *p, const Flow *f, JsonBuilder *js)
 {
     if (cfg->include_metadata) {
-        JsonAddMetadata(p, f, js);
+        EveAddMetadata(p, f, js);
+    }
+    if (cfg->include_ethernet) {
+        CreateJSONEther(js, p, f);
     }
     if (cfg->include_community_id && f != NULL) {
-        CreateJSONCommunityFlowId(js, f, cfg->community_id_seed);
+        CreateEveCommunityFlowId(js, f, cfg->community_id_seed);
     }
 }
 
@@ -405,58 +462,51 @@ void JsonAddCommonOptions(const OutputJsonCommonSettings *cfg,
  * \param js JSON object
  * \param max_length If non-zero, restricts the number of packet data bytes handled.
  */
-void JsonPacket(const Packet *p, json_t *js, unsigned long max_length)
+void EvePacket(const Packet *p, JsonBuilder *js, unsigned long max_length)
 {
     unsigned long max_len = max_length == 0 ? GET_PKT_LEN(p) : max_length;
-    unsigned long len = 2 * max_len;
+    unsigned long len = BASE64_BUFFER_SIZE(max_len);
     uint8_t encoded_packet[len];
     if (Base64Encode((unsigned char*) GET_PKT_DATA(p), max_len, encoded_packet, &len) == SC_BASE64_OK) {
-        json_object_set_new(js, "packet", json_string((char *)encoded_packet));
+        jb_set_string(js, "packet", (char *)encoded_packet);
     }
 
-    /* Create packet info. */
-    json_t *packetinfo_js = json_object();
-    if (unlikely(packetinfo_js == NULL)) {
+    if (!jb_open_object(js, "packet_info")) {
         return;
     }
-    json_object_set_new(packetinfo_js, "linktype", json_integer(p->datalink));
-    json_object_set_new(js, "packet_info", packetinfo_js);
+    if (!jb_set_uint(js, "linktype", p->datalink)) {
+        return;
+    }
+    jb_close(js);
 }
+
 /** \brief jsonify tcp flags field
  *  Only add 'true' fields in an attempt to keep things reasonably compact.
  */
-void JsonTcpFlags(uint8_t flags, json_t *js)
+void EveTcpFlags(const uint8_t flags, JsonBuilder *js)
 {
     if (flags & TH_SYN)
-        json_object_set_new(js, "syn", json_true());
+        JB_SET_TRUE(js, "syn");
     if (flags & TH_FIN)
-        json_object_set_new(js, "fin", json_true());
+        JB_SET_TRUE(js, "fin");
     if (flags & TH_RST)
-        json_object_set_new(js, "rst", json_true());
+        JB_SET_TRUE(js, "rst");
     if (flags & TH_PUSH)
-        json_object_set_new(js, "psh", json_true());
+        JB_SET_TRUE(js, "psh");
     if (flags & TH_ACK)
-        json_object_set_new(js, "ack", json_true());
+        JB_SET_TRUE(js, "ack");
     if (flags & TH_URG)
-        json_object_set_new(js, "urg", json_true());
+        JB_SET_TRUE(js, "urg");
     if (flags & TH_ECN)
-        json_object_set_new(js, "ecn", json_true());
+        JB_SET_TRUE(js, "ecn");
     if (flags & TH_CWR)
-        json_object_set_new(js, "cwr", json_true());
+        JB_SET_TRUE(js, "cwr");
 }
 
-/**
- * \brief Add five tuple from packet to JSON object
- *
- * \param p Packet
- * \param dir log direction (packet or flow)
- * \param js JSON object
- */
-void JsonFiveTuple(const Packet *p, enum OutputJsonLogDirection dir, json_t *js)
+void JsonAddrInfoInit(const Packet *p, enum OutputJsonLogDirection dir, JsonAddrInfo *addr)
 {
     char srcip[46] = {0}, dstip[46] = {0};
     Port sp, dp;
-    char proto[16];
 
     switch (dir) {
         case LOG_DIR_PACKET:
@@ -545,41 +595,42 @@ void JsonFiveTuple(const Packet *p, enum OutputJsonLogDirection dir, json_t *js)
             return;
     }
 
-    if (SCProtoNameValid(IP_GET_IPPROTO(p)) == TRUE) {
-        strlcpy(proto, known_proto[IP_GET_IPPROTO(p)], sizeof(proto));
+
+    strlcpy(addr->src_ip, srcip, JSON_ADDR_LEN);
+
+    switch(p->proto) {
+        case IPPROTO_ICMP:
+            break;
+        case IPPROTO_UDP:
+        case IPPROTO_TCP:
+        case IPPROTO_SCTP:
+            addr->sp = sp;
+            break;
+    }
+
+    strlcpy(addr->dst_ip, dstip, JSON_ADDR_LEN);
+
+    switch(p->proto) {
+        case IPPROTO_ICMP:
+            break;
+        case IPPROTO_UDP:
+        case IPPROTO_TCP:
+        case IPPROTO_SCTP:
+            addr->dp = dp;
+            break;
+    }
+
+    if (SCProtoNameValid(IP_GET_IPPROTO(p))) {
+        strlcpy(addr->proto, known_proto[IP_GET_IPPROTO(p)], sizeof(addr->proto));
     } else {
-        snprintf(proto, sizeof(proto), "%03" PRIu32, IP_GET_IPPROTO(p));
+        snprintf(addr->proto, sizeof(addr->proto), "%" PRIu32, IP_GET_IPPROTO(p));
     }
-
-    json_object_set_new(js, "src_ip", json_string(srcip));
-
-    switch(p->proto) {
-        case IPPROTO_ICMP:
-            break;
-        case IPPROTO_UDP:
-        case IPPROTO_TCP:
-        case IPPROTO_SCTP:
-            json_object_set_new(js, "src_port", json_integer(sp));
-            break;
-    }
-
-    json_object_set_new(js, "dest_ip", json_string(dstip));
-
-    switch(p->proto) {
-        case IPPROTO_ICMP:
-            break;
-        case IPPROTO_UDP:
-        case IPPROTO_TCP:
-        case IPPROTO_SCTP:
-            json_object_set_new(js, "dest_port", json_integer(dp));
-            break;
-    }
-
-    json_object_set_new(js, "proto", json_string(proto));
 }
 
-static void CreateJSONCommunityFlowIdv4(json_t *js, const Flow *f,
-        const uint16_t seed)
+#define COMMUNITY_ID_BUF_SIZE 64
+
+static bool CalculateCommunityFlowIdv4(const Flow *f,
+        const uint16_t seed, unsigned char *base64buf)
 {
     struct {
         uint16_t seed;
@@ -619,12 +670,13 @@ static void CreateJSONCommunityFlowIdv4(json_t *js, const Flow *f,
 
     uint8_t hash[20];
     if (ComputeSHA1((const uint8_t *)&ipv4, sizeof(ipv4), hash, sizeof(hash)) == 1) {
-        unsigned char base64buf[64] = "1:";
-        unsigned long out_len = sizeof(base64buf) - 2;
+        strlcpy((char *)base64buf, "1:", COMMUNITY_ID_BUF_SIZE);
+        unsigned long out_len = COMMUNITY_ID_BUF_SIZE - 2;
         if (Base64Encode(hash, sizeof(hash), base64buf+2, &out_len) == SC_BASE64_OK) {
-            json_object_set_new(js, "community_id", json_string((const char *)base64buf));
+            return true;
         }
     }
+    return false;
 }
 
 static inline bool FlowHashRawAddressIPv6LtU32(const uint32_t *a, const uint32_t *b)
@@ -639,8 +691,8 @@ static inline bool FlowHashRawAddressIPv6LtU32(const uint32_t *a, const uint32_t
     return false;
 }
 
-static void CreateJSONCommunityFlowIdv6(json_t *js, const Flow *f,
-        const uint16_t seed)
+static bool CalculateCommunityFlowIdv6(const Flow *f,
+        const uint16_t seed, unsigned char *base64buf)
 {
     struct {
         uint16_t seed;
@@ -680,99 +732,194 @@ static void CreateJSONCommunityFlowIdv6(json_t *js, const Flow *f,
 
     uint8_t hash[20];
     if (ComputeSHA1((const uint8_t *)&ipv6, sizeof(ipv6), hash, sizeof(hash)) == 1) {
-        unsigned char base64buf[64] = "1:";
-        unsigned long out_len = sizeof(base64buf) - 2;
+        strlcpy((char *)base64buf, "1:", COMMUNITY_ID_BUF_SIZE);
+        unsigned long out_len = COMMUNITY_ID_BUF_SIZE - 2;
         if (Base64Encode(hash, sizeof(hash), base64buf+2, &out_len) == SC_BASE64_OK) {
-            json_object_set_new(js, "community_id", json_string((const char *)base64buf));
+            return true;
+        }
+    }
+    return false;
+}
+
+static void CreateEveCommunityFlowId(JsonBuilder *js, const Flow *f, const uint16_t seed)
+{
+    unsigned char buf[COMMUNITY_ID_BUF_SIZE];
+    if (f->flags & FLOW_IPV4) {
+        if (CalculateCommunityFlowIdv4(f, seed, buf)) {
+            jb_set_string(js, "community_id", (const char *)buf);
+        }
+    } else if (f->flags & FLOW_IPV6) {
+        if (CalculateCommunityFlowIdv6(f, seed, buf)) {
+            jb_set_string(js, "community_id", (const char *)buf);
         }
     }
 }
 
-static void CreateJSONCommunityFlowId(json_t *js, const Flow *f, const uint16_t seed)
+void CreateEveFlowId(JsonBuilder *js, const Flow *f)
 {
-    if (f->flags & FLOW_IPV4)
-        return CreateJSONCommunityFlowIdv4(js, f, seed);
-    else if (f->flags & FLOW_IPV6)
-        return CreateJSONCommunityFlowIdv6(js, f, seed);
-}
-
-void CreateJSONFlowId(json_t *js, const Flow *f)
-{
-    if (f == NULL)
+    if (f == NULL) {
         return;
+    }
     int64_t flow_id = FlowGetId(f);
-    json_object_set_new(js, "flow_id", json_integer(flow_id));
+    jb_set_uint(js, "flow_id", flow_id);
     if (f->parent_id) {
-        json_object_set_new(js, "parent_id", json_integer(f->parent_id));
+        jb_set_uint(js, "parent_id", f->parent_id);
     }
 }
 
-json_t *CreateJSONHeader(const Packet *p, enum OutputJsonLogDirection dir,
-                         const char *event_type)
+static inline void JSONFormatAndAddMACAddr(JsonBuilder *js, const char *key,
+                                   uint8_t *val, bool is_array)
+{
+    char eth_addr[19];
+    (void) snprintf(eth_addr, 19, "%02x:%02x:%02x:%02x:%02x:%02x",
+                    val[0], val[1], val[2], val[3], val[4], val[5]);
+    if (is_array) {
+        jb_append_string(js, eth_addr);
+    } else {
+        jb_set_string(js, key, eth_addr);
+    }
+}
+
+/* only required to traverse the MAC address set */
+typedef struct JSONMACAddrInfo {
+    JsonBuilder *src, *dst;
+} JSONMACAddrInfo;
+
+static int MacSetIterateToJSON(uint8_t *val, MacSetSide side, void *data)
+{
+    JSONMACAddrInfo *info = (JSONMACAddrInfo*) data;
+    if (side == MAC_SET_DST) {
+        JSONFormatAndAddMACAddr(info->dst, NULL, val, true);
+    } else {
+        JSONFormatAndAddMACAddr(info->src, NULL, val, true);
+    }
+    return 0;
+}
+
+int CreateJSONEther(JsonBuilder *js, const Packet *p, const Flow *f)
+{
+    if (unlikely(js == NULL))
+        return 0;
+    /* start new EVE sub-object */
+    jb_open_object(js, "ether");
+    if (p == NULL) {
+        MacSet *ms = NULL;
+        /* ensure we have a flow */
+        if (unlikely(f == NULL)) {
+            jb_close(js);
+            return 0;
+        }
+        /* we are creating an ether object in a flow context, so we need to
+           append to arrays */
+        ms = FlowGetStorageById((Flow *)f, MacSetGetFlowStorageID());
+        if (ms != NULL && MacSetSize(ms) > 0) {
+            JSONMACAddrInfo info;
+            info.dst = jb_new_array();
+            info.src = jb_new_array();
+            int ret = MacSetForEach(ms, MacSetIterateToJSON, &info);
+            if (unlikely(ret != 0)) {
+                /* should not happen, JSONFlowAppendMACAddrs is sane */
+                jb_free(info.dst);
+                jb_free(info.src);
+                jb_close(js);
+                return ret;
+            }
+            jb_close(info.dst);
+            jb_close(info.src);
+            jb_set_object(js, "dest_macs", info.dst);
+            jb_set_object(js, "src_macs", info.src);
+            jb_free(info.dst);
+            jb_free(info.src);
+        }
+    } else {
+        /* this is a packet context, so we need to add scalar fields */
+        uint8_t *src, *dst;
+        if (p->ethh != NULL) {
+            if ((PKT_IS_TOCLIENT(p))) {
+                src = p->ethh->eth_dst;
+                dst = p->ethh->eth_src;
+            } else {
+                src = p->ethh->eth_src;
+                dst = p->ethh->eth_dst;
+            }
+            JSONFormatAndAddMACAddr(js, "src_mac", src, false);
+            JSONFormatAndAddMACAddr(js, "dest_mac", dst, false);
+        }
+    }
+    jb_close(js);
+    return 0;
+}
+
+JsonBuilder *CreateEveHeader(const Packet *p, enum OutputJsonLogDirection dir,
+        const char *event_type, JsonAddrInfo *addr)
 {
     char timebuf[64];
     const Flow *f = (const Flow *)p->flow;
 
-    json_t *js = json_object();
-    if (unlikely(js == NULL))
+    JsonBuilder *js = jb_new_object();
+    if (unlikely(js == NULL)) {
         return NULL;
+    }
 
     CreateIsoTimeString(&p->ts, timebuf, sizeof(timebuf));
 
-    /* time & tx */
-    json_object_set_new(js, "timestamp", json_string(timebuf));
+    jb_set_string(js, "timestamp", timebuf);
 
-    CreateJSONFlowId(js, f);
+    CreateEveFlowId(js, f);
 
     /* sensor id */
-    if (sensor_id >= 0)
-        json_object_set_new(js, "sensor_id", json_integer(sensor_id));
+    if (sensor_id >= 0) {
+        jb_set_uint(js, "sensor_id", sensor_id);
+    }
 
     /* input interface */
     if (p->livedev) {
-        json_object_set_new(js, "in_iface", json_string(p->livedev->dev));
+        jb_set_string(js, "in_iface", p->livedev->dev);
     }
 
     /* pcap_cnt */
     if (p->pcap_cnt != 0) {
-        json_object_set_new(js, "pcap_cnt", json_integer(p->pcap_cnt));
+        jb_set_uint(js, "pcap_cnt", p->pcap_cnt);
     }
 
     if (event_type) {
-        json_object_set_new(js, "event_type", json_string(event_type));
+        jb_set_string(js, "event_type", event_type);
     }
 
     /* vlan */
     if (p->vlan_idx > 0) {
-        json_t *js_vlan = json_array();
-        if (js_vlan) {
-            json_array_append_new(js_vlan, json_integer(p->vlan_id[0]));
-            if (p->vlan_idx > 1) {
-                json_array_append_new(js_vlan, json_integer(p->vlan_id[1]));
-            }
-            json_object_set_new(js, "vlan", js_vlan);
+        jb_open_array(js, "vlan");
+        jb_append_uint(js, p->vlan_id[0]);
+        if (p->vlan_idx > 1) {
+            jb_append_uint(js, p->vlan_id[1]);
         }
+        jb_close(js);
     }
 
     /* 5-tuple */
-    JsonFiveTuple(p, dir, js);
+    JsonAddrInfo addr_info = json_addr_info_zero;
+    if (addr == NULL) {
+        JsonAddrInfoInit(p, dir, &addr_info);
+        addr = &addr_info;
+    }
+    jb_set_string(js, "src_ip", addr->src_ip);
+    jb_set_uint(js, "src_port", addr->sp);
+    jb_set_string(js, "dest_ip", addr->dst_ip);
+    jb_set_uint(js, "dest_port", addr->dp);
+    jb_set_string(js, "proto", addr->proto);
 
     /* icmp */
     switch (p->proto) {
         case IPPROTO_ICMP:
             if (p->icmpv4h) {
-                json_object_set_new(js, "icmp_type",
-                                    json_integer(p->icmpv4h->type));
-                json_object_set_new(js, "icmp_code",
-                                    json_integer(p->icmpv4h->code));
+                jb_set_uint(js, "icmp_type", p->icmpv4h->type);
+                jb_set_uint(js, "icmp_code", p->icmpv4h->code);
             }
             break;
         case IPPROTO_ICMPV6:
             if (p->icmpv6h) {
-                json_object_set_new(js, "icmp_type",
-                                    json_integer(p->icmpv6h->type));
-                json_object_set_new(js, "icmp_code",
-                                    json_integer(p->icmpv6h->code));
+                jb_set_uint(js, "icmp_type", p->icmpv6h->type);
+                jb_set_uint(js, "icmp_code", p->icmpv6h->code);
             }
             break;
     }
@@ -780,15 +927,15 @@ json_t *CreateJSONHeader(const Packet *p, enum OutputJsonLogDirection dir,
     return js;
 }
 
-json_t *CreateJSONHeaderWithTxId(const Packet *p, enum OutputJsonLogDirection dir,
-                                 const char *event_type, uint64_t tx_id)
+JsonBuilder *CreateEveHeaderWithTxId(const Packet *p, enum OutputJsonLogDirection dir,
+                                 const char *event_type, JsonAddrInfo *addr, uint64_t tx_id)
 {
-    json_t *js = CreateJSONHeader(p, dir, event_type);
+    JsonBuilder *js = CreateEveHeader(p, dir, event_type, addr);
     if (unlikely(js == NULL))
         return NULL;
 
     /* tx id for correlation with other events */
-    json_object_set_new(js, "tx_id", json_integer(tx_id));
+    jb_set_uint(js, "tx_id", tx_id);
 
     return js;
 }
@@ -835,6 +982,33 @@ int OutputJSONBuffer(json_t *js, LogFileCtx *file_ctx, MemBuffer **buffer)
     return 0;
 }
 
+int OutputJsonBuilderBuffer(JsonBuilder *js, LogFileCtx *file_ctx, MemBuffer **buffer)
+{
+    if (file_ctx->sensor_name) {
+        jb_set_string(js, "host", file_ctx->sensor_name);
+    }
+
+    if (file_ctx->is_pcap_offline) {
+        jb_set_string(js, "pcap_filename", PcapFileGetFilename());
+    }
+
+    jb_close(js);
+
+    if (file_ctx->prefix) {
+        MemBufferWriteRaw((*buffer), file_ctx->prefix, file_ctx->prefix_len);
+    }
+
+    size_t jslen = jb_len(js);
+    if (MEMBUFFER_OFFSET(*buffer) + jslen >= MEMBUFFER_SIZE(*buffer)) {
+        MemBufferExpand(buffer, jslen);
+    }
+
+    MemBufferWriteRaw((*buffer), jb_ptr(js), jslen);
+    LogFileWrite(file_ctx, *buffer);
+
+    return 0;
+}
+
 /**
  * \brief Create a new LogFileCtx for "fast" output style.
  * \param conf The configuration node for this output.
@@ -913,14 +1087,23 @@ OutputInitResult OutputJsonInitCtx(ConfNode *conf)
                 SCLogRedisInit();
                 json_ctx->json_out = LOGFILE_TYPE_REDIS;
 #else
-                SCLogError(SC_ERR_INVALID_ARGUMENT,
-                           "redis JSON output option is not compiled");
-                exit(EXIT_FAILURE);
+                           FatalError(SC_ERR_FATAL,
+                                      "redis JSON output option is not compiled");
 #endif
             } else {
-                SCLogError(SC_ERR_INVALID_ARGUMENT,
-                           "Invalid JSON output option: %s", output_s);
-                exit(EXIT_FAILURE);
+#ifdef HAVE_PLUGINS
+                SCPluginFileType *plugin = SCPluginFindFileType(output_s);
+                if (plugin == NULL) {
+                    FatalError(SC_ERR_INVALID_ARGUMENT,
+                            "Invalid JSON output option: %s", output_s);
+                } else {
+                    json_ctx->json_out = LOGFILE_TYPE_PLUGIN;
+                    json_ctx->plugin = plugin;
+                }
+#else
+                FatalError(SC_ERR_INVALID_ARGUMENT,
+                        "Invalid JSON output option: %s", output_s);
+#endif
             }
         }
 
@@ -931,9 +1114,8 @@ OutputInitResult OutputJsonInitCtx(ConfNode *conf)
             json_ctx->file_ctx->prefix = SCStrdup(prefix);
             if (json_ctx->file_ctx->prefix == NULL)
             {
-                SCLogError(SC_ERR_MEM_ALLOC,
-                    "Failed to allocate memory for eve-log.prefix setting.");
-                exit(EXIT_FAILURE);
+                    FatalError(SC_ERR_FATAL,
+                               "Failed to allocate memory for eve-log.prefix setting.");
             }
             json_ctx->file_ctx->prefix_len = strlen(prefix);
         }
@@ -942,6 +1124,17 @@ OutputInitResult OutputJsonInitCtx(ConfNode *conf)
             json_ctx->json_out == LOGFILE_TYPE_UNIX_DGRAM ||
             json_ctx->json_out == LOGFILE_TYPE_UNIX_STREAM)
         {
+            if (json_ctx->json_out == LOGFILE_TYPE_FILE) {
+                /* Threaded file output */
+                const ConfNode *threaded = ConfNodeLookupChild(conf, "threaded");
+                if (threaded && threaded->val && ConfValIsTrue(threaded->val)) {
+                    SCLogConfig("Enabling threaded eve logging.");
+                    json_ctx->file_ctx->threaded = true;
+                } else {
+                    json_ctx->file_ctx->threaded = false;
+                }
+            }
+
             if (SCConfLogOpenGeneric(conf, json_ctx->file_ctx, DEFAULT_LOG_FILENAME, 1) < 0) {
                 LogFileFreeCtx(json_ctx->file_ctx);
                 SCFree(json_ctx);
@@ -949,6 +1142,7 @@ OutputInitResult OutputJsonInitCtx(ConfNode *conf)
                 return result;
             }
             OutputRegisterFileRotationFlag(&json_ctx->file_ctx->rotation_flag);
+
         }
 #ifndef OS_WIN32
 	else if (json_ctx->json_out == LOGFILE_TYPE_SYSLOG) {
@@ -1003,10 +1197,24 @@ OutputInitResult OutputJsonInitCtx(ConfNode *conf)
             }
         }
 #endif
+        else if (json_ctx->json_out == LOGFILE_TYPE_PLUGIN) {
+            ConfNode *plugin_conf = ConfNodeLookupChild(conf,
+                json_ctx->plugin->name);
+            void *plugin_data = NULL;
+            if (json_ctx->plugin->Open(plugin_conf, &plugin_data) < 0) {
+                LogFileFreeCtx(json_ctx->file_ctx);
+                SCFree(json_ctx);
+                SCFree(output_ctx);
+                return result;
+            } else {
+                json_ctx->file_ctx->plugin = json_ctx->plugin;
+                json_ctx->file_ctx->plugin_data = plugin_data;
+            }
+        }
 
         const char *sensor_id_s = ConfNodeLookupChildValue(conf, "sensor-id");
         if (sensor_id_s != NULL) {
-            if (ByteExtractStringUint64((uint64_t *)&sensor_id, 10, 0, sensor_id_s) == -1) {
+            if (StringParseUint64((uint64_t *)&sensor_id, 10, 0, sensor_id_s) < 0) {
                 SCLogError(SC_ERR_INVALID_ARGUMENT,
                            "Failed to initialize JSON output, "
                            "invalid sensor-id: %s", sensor_id_s);
@@ -1023,6 +1231,15 @@ OutputInitResult OutputJsonInitCtx(ConfNode *conf)
             json_ctx->cfg.include_metadata = true;
         }
 
+        /* Check if ethernet information should be logged. */
+        const ConfNode *ethernet = ConfNodeLookupChild(conf, "ethernet");
+        if (ethernet && ethernet->val && ConfValIsTrue(ethernet->val)) {
+            SCLogConfig("Enabling Ethernet MAC address logging.");
+            json_ctx->cfg.include_ethernet = true;
+        } else {
+            json_ctx->cfg.include_ethernet = false;
+        }
+
         /* See if we want to enable the community id */
         const ConfNode *community_id = ConfNodeLookupChild(conf, "community-id");
         if (community_id && community_id->val && ConfValIsTrue(community_id->val)) {
@@ -1033,8 +1250,8 @@ OutputInitResult OutputJsonInitCtx(ConfNode *conf)
         }
         const char *cid_seed = ConfNodeLookupChildValue(conf, "community-id-seed");
         if (cid_seed != NULL) {
-            if (ByteExtractStringUint16(&json_ctx->cfg.community_id_seed,
-                        10, 0, cid_seed) == -1)
+            if (StringParseUint16(&json_ctx->cfg.community_id_seed,
+                        10, 0, cid_seed) < 0)
             {
                 SCLogError(SC_ERR_INVALID_ARGUMENT,
                            "Failed to initialize JSON output, "
@@ -1062,7 +1279,6 @@ OutputInitResult OutputJsonInitCtx(ConfNode *conf)
         json_ctx->file_ctx->type = json_ctx->json_out;
     }
 
-
     SCLogDebug("returning output_ctx %p", output_ctx);
 
     result.ctx = output_ctx;
