@@ -31,7 +31,6 @@
 */
 
 #include "suricata-common.h"
-#include "config.h"
 #include "tm-threads.h"
 #include "conf.h"
 #include "runmodes.h"
@@ -42,7 +41,6 @@
 
 #include "alert-fastlog.h"
 #include "alert-prelude.h"
-#include "alert-unified2-alert.h"
 #include "alert-debuglog.h"
 
 #include "util-debug.h"
@@ -52,6 +50,7 @@
 #include "util-device.h"
 #include "util-runmodes.h"
 #include "util-ioctl.h"
+#include "util-byte.h"
 
 #include "source-netmap.h"
 
@@ -85,7 +84,7 @@ static void NetmapDerefConfig(void *conf)
 {
     NetmapIfaceConfig *pfp = (NetmapIfaceConfig *)conf;
     /* config is used only once but cost of this low. */
-    if (SC_ATOMIC_SUB(pfp->ref, 1) == 0) {
+    if (SC_ATOMIC_SUB(pfp->ref, 1) == 1) {
         SCFree(pfp);
     }
 }
@@ -149,7 +148,11 @@ static int ParseNetmapSettings(NetmapIfaceSettings *ns, const char *iface,
             ns->threads = 0;
             ns->threads_auto = true;
         } else {
-            ns->threads = atoi(threadsstr);
+            if (StringParseUint16(&ns->threads, 10, 0, threadsstr) < 0) {
+                SCLogWarning(SC_ERR_INVALID_VALUE, "Invalid config value for "
+                             "threads: %s, resetting to 0", threadsstr);
+                ns->threads = 0;
+            }
         }
     }
 
@@ -419,8 +422,7 @@ int RunModeIdsNetmapAutoFp(void)
                               "DecodeNetmap", thread_name_autofp,
                               live_dev);
     if (ret != 0) {
-        SCLogError(SC_ERR_RUNMODE, "Unable to start runmode");
-        exit(EXIT_FAILURE);
+        FatalError(SC_ERR_FATAL, "Unable to start runmode");
     }
 
     SCLogDebug("RunModeIdsNetmapAutoFp initialised");
@@ -452,8 +454,7 @@ int RunModeIdsNetmapSingle(void)
                                     "DecodeNetmap", thread_name_single,
                                     live_dev);
     if (ret != 0) {
-        SCLogError(SC_ERR_RUNMODE, "Unable to start runmode");
-        exit(EXIT_FAILURE);
+        FatalError(SC_ERR_FATAL, "Unable to start runmode");
     }
 
     SCLogDebug("RunModeIdsNetmapSingle initialised");
@@ -488,8 +489,7 @@ int RunModeIdsNetmapWorkers(void)
                                     "DecodeNetmap", thread_name_workers,
                                     live_dev);
     if (ret != 0) {
-        SCLogError(SC_ERR_RUNMODE, "Unable to start runmode");
-        exit(EXIT_FAILURE);
+        FatalError(SC_ERR_FATAL, "Unable to start runmode");
     }
 
     SCLogDebug("RunModeIdsNetmapWorkers initialised");
