@@ -37,7 +37,7 @@
 #define __USE_GNU
 
 #if HAVE_CONFIG_H
-#include <config.h>
+#include <autoconf.h>
 #endif
 
 #ifndef CLS
@@ -125,6 +125,19 @@
 #include <sched.h>     /* for sched_setaffinity(2) */
 #endif
 
+#ifdef HAVE_TYPE_U_LONG_NOT_DEFINED
+typedef unsigned long int u_long
+#endif
+#ifdef HAVE_TYPE_U_INT_NOT_DEFINED
+typedef unsigned int u_int
+#endif
+#ifdef HAVE_TYPE_U_SHORT_NOT_DEFINED
+typedef unsigned short u_short
+#endif
+#ifdef HAVE_TYPE_U_CHAR_NOT_DEFINED
+typedef unsigned char u_char
+#endif
+
 #include <pcre.h>
 
 #ifdef HAVE_SYSLOG_H
@@ -156,10 +169,6 @@
 #include <signal.h>
 #endif
 
-#if HAVE_SYS_TYPES_H
-#include <sys/types.h>
-#endif
-
 #if HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
 #endif
@@ -188,24 +197,6 @@
 #include <netdb.h>
 #endif
 
-#ifndef SC_PCAP_DONT_INCLUDE_PCAP_H
-#ifdef HAVE_PCAP_H
-#include <pcap.h>
-#endif
-
-#ifdef HAVE_PCAP_PCAP_H
-#include <pcap/pcap.h>
-#endif
-#endif
-
-#ifdef HAVE_UTIME_H
-#include <utime.h>
-#endif
-
-#ifdef HAVE_LIBGEN_H
-#include <libgen.h>
-#endif
-
 #if __CYGWIN__
 #if !defined _X86_ && !defined __x86_64
 #define _X86_
@@ -236,6 +227,32 @@
 #include <w32api/wtypes.h>
 #endif
 
+#ifndef SC_PCAP_DONT_INCLUDE_PCAP_H
+#ifdef HAVE_PCAP_H
+#include <pcap.h>
+#endif
+
+#ifdef HAVE_PCAP_PCAP_H
+#include <pcap/pcap.h>
+#endif
+#endif
+
+#ifdef HAVE_UTIME_H
+#include <utime.h>
+#endif
+
+#ifdef HAVE_LIBGEN_H
+#include <libgen.h>
+#endif
+
+#ifdef HAVE_GRP_H
+#include <grp.h>
+#endif
+
+#ifdef HAVE_PWD_H
+#include <pwd.h>
+#endif
+
 #include <jansson.h>
 #ifndef JSON_ESCAPE_SLASH
 #define JSON_ESCAPE_SLASH 0
@@ -371,6 +388,14 @@
 
 #define WARN_UNUSED __attribute__((warn_unused_result))
 
+#if defined(__MINGW32__)
+#define ATTR_FMT_PRINTF(x, y) __attribute__((format(__MINGW_PRINTF_FORMAT, (x), (y))))
+#elif defined(__GNUC__)
+#define ATTR_FMT_PRINTF(x, y) __attribute__((format(printf, (x), (y))))
+#else
+#define ATTR_FMT_PRINTF(x, y)
+#endif
+
 #define SCNtohl(x) (uint32_t)ntohl((x))
 #define SCNtohs(x) (uint16_t)ntohs((x))
 
@@ -440,8 +465,12 @@ typedef enum {
     LOGGER_JSON_SNMP,
     LOGGER_JSON_SIP,
     LOGGER_JSON_TEMPLATE_RUST,
+    LOGGER_JSON_RFB,
+    LOGGER_JSON_MQTT,
     LOGGER_JSON_TEMPLATE,
     LOGGER_JSON_RDP,
+    LOGGER_JSON_DCERPC,
+    LOGGER_JSON_HTTP2,
 
     LOGGER_ALERT_DEBUG,
     LOGGER_ALERT_FAST,
@@ -465,13 +494,17 @@ typedef enum {
 } LoggerId;
 
 #include "util-optimize.h"
+#ifndef SURICATA_PLUGIN
 #include <htp/htp.h>
+#endif
 #include "threads.h"
 #include "tm-threads-common.h"
 #include "util-debug.h"
 #include "util-error.h"
 #include "util-mem.h"
+#ifndef SURICATA_PLUGIN
 #include "detect-engine-alert.h"
+#endif
 #include "util-path.h"
 #include "util-conf.h"
 
@@ -479,6 +512,10 @@ typedef enum {
 #include <lua.h>
 #include <lualib.h>
 #include <lauxlib.h>
+#else
+/* If we don't have Lua, create a typedef for lua_State so the
+ * exported Lua functions don't fail the build. */
+typedef void lua_State;
 #endif
 
 #ifndef HAVE_STRLCAT
@@ -491,6 +528,17 @@ size_t strlcpy(char *dst, const char *src, size_t siz);
 char *strptime(const char * __restrict, const char * __restrict, struct tm * __restrict);
 #endif
 
+#ifndef HAVE_FWRITE_UNLOCKED
+#define SCFwriteUnlocked    fwrite
+#define SCFflushUnlocked    fflush
+#define SCClearErrUnlocked  clearerr
+#define SCFerrorUnlocked    ferror
+#else
+#define SCFwriteUnlocked    fwrite_unlocked
+#define SCFflushUnlocked    fflush_unlocked
+#define SCClearErrUnlocked  clearerr_unlocked
+#define SCFerrorUnlocked    ferror_unlocked
+#endif
 extern int coverage_unittests;
 extern int g_ut_modules;
 extern int g_ut_covered;
