@@ -189,6 +189,13 @@ static int DetectEngineContentInspectionTest08(void) {
     TEST_RUN("ababc", 5, "content:\"a\"; content:\"b\"; distance:0; within:1; content:!\"a\"; distance:0; within:1;", true, 5);
     TEST_RUN("ababc", 5, "content:\"a\"; content:\"b\"; distance:0; within:1; content:!\"a\"; distance:0; ", true, 5);
 
+    TEST_RUN("abcdefghy", 9, "content:\"a\"; content:!\"x\"; content:\"c\"; distance:0; within:2; ",
+            true, 3);
+    TEST_RUN("abcdefghx", 9, "content:\"a\"; content:!\"x\"; content:\"c\"; distance:0; within:2; ",
+            false, 2);
+    TEST_RUN("abcdefghy", 9,
+            "content:\"a\"; content:!\"x\"; content:!\"c\"; distance:2; within:1; ", true, 3);
+
     TEST_FOOTER;
 }
 
