@@ -1,4 +1,4 @@
-/* Copyright (C) 2007-2010 Open Information Security Foundation
+/* Copyright (C) 2007-2020 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -49,6 +49,7 @@ const char * SCErrorToString(SCError err)
         CASE_CODE (SC_ERR_PCRE_COMPILE);
         CASE_CODE (SC_ERR_PCRE_STUDY);
         CASE_CODE (SC_ERR_PCRE_PARSE);
+        CASE_CODE (SC_WARN_PCRE_JITSTACK);
         CASE_CODE (SC_ERR_LOG_MODULE_NOT_INIT);
         CASE_CODE (SC_ERR_LOG_FG_FILTER_MATCH);
         CASE_CODE (SC_ERR_PCAP_DISPATCH);
@@ -107,6 +108,7 @@ const char * SCErrorToString(SCError err)
         CASE_CODE (SC_ERR_DEBUG_LOG_GENERIC);
         CASE_CODE (SC_ERR_UNIFIED_LOG_GENERIC);
         CASE_CODE (SC_ERR_HTTP_LOG_GENERIC);
+        CASE_CODE (SC_ERR_HTTP2_LOG_GENERIC);
         CASE_CODE (SC_ERR_FTP_LOG_GENERIC);
         CASE_CODE (SC_ERR_UNIFIED_ALERT_GENERIC);
         CASE_CODE (SC_ERR_UNIFIED2_ALERT_GENERIC);
@@ -366,6 +368,15 @@ const char * SCErrorToString(SCError err)
         CASE_CODE (SC_ERR_DATASET);
         CASE_CODE (SC_WARN_ANOMALY_CONFIG);
         CASE_CODE (SC_WARN_ALERT_CONFIG);
+        CASE_CODE (SC_WARN_REGISTRATION_FAILED);
+        CASE_CODE (SC_ERR_ERF_BAD_RLEN);
+        CASE_CODE (SC_WARN_ERSPAN_CONFIG);
+        CASE_CODE (SC_WARN_HASSH_DISABLED);
+        CASE_CODE (SC_WARN_FILESTORE_CONFIG);
+        CASE_CODE (SC_WARN_PATH_READ_ERROR);
+        CASE_CODE (SC_ERR_PLUGIN);
+        CASE_CODE(SC_ERR_LOG_OUTPUT);
+        CASE_CODE(SC_ERR_RULE_INVALID_UTF8);
 
         CASE_CODE (SC_ERR_MAX);
     }
