@@ -30,6 +30,7 @@
 
 #include "flow-private.h"
 #include "flow-util.h"
+#include "flow-spare-pool.h"
 
 #include "detect.h"
 #include "detect-parse.h"
@@ -45,6 +46,80 @@
 #include "util-unittest.h"
 #include "util-unittest-helper.h"
 
+#if defined(UNITTESTS) || defined(FUZZ)
+Flow *TestHelperBuildFlow(int family, const char *src, const char *dst, Port sp, Port dp)
+{
+    struct in_addr in;
+
+    Flow *f = SCMalloc(sizeof(Flow));
+    if (unlikely(f == NULL)) {
+        printf("FlowAlloc failed\n");
+        ;
+        return NULL;
+    }
+    memset(f, 0x00, sizeof(Flow));
+
+    FLOW_INITIALIZE(f);
+
+    if (family == AF_INET) {
+        f->flags |= FLOW_IPV4;
+    } else if (family == AF_INET6) {
+        f->flags |= FLOW_IPV6;
+    }
+
+    if (src != NULL) {
+        if (family == AF_INET) {
+            if (inet_pton(AF_INET, src, &in) != 1) {
+                printf("invalid address %s\n", src);
+                SCFree(f);
+                return NULL;
+            }
+            f->src.addr_data32[0] = in.s_addr;
+        } else {
+            BUG_ON(1);
+        }
+    }
+    if (dst != NULL) {
+        if (family == AF_INET) {
+            if (inet_pton(AF_INET, dst, &in) != 1) {
+                printf("invalid address %s\n", dst);
+                SCFree(f);
+                return NULL;
+            }
+            f->dst.addr_data32[0] = in.s_addr;
+        } else {
+            BUG_ON(1);
+        }
+    }
+
+    f->sp = sp;
+    f->dp = dp;
+
+    return f;
+}
+/** \brief writes the contents of a buffer into a file */
+int TestHelperBufferToFile(const char *name, const uint8_t *data, size_t size)
+{
+    if (remove(name) != 0) {
+        if (errno != ENOENT) {
+            printf("failed remove, errno=%d\n", errno);
+            return -1;
+        }
+    }
+    FILE *fd = fopen(name, "wb");
+    if (fd == NULL) {
+        printf("failed open, errno=%d\n", errno);
+        return -2;
+    }
+    if (fwrite (data, 1, size, fd) != size) {
+        fclose(fd);
+        return -3;
+    }
+    fclose(fd);
+    return 0;
+}
+
+#endif
 #ifdef UNITTESTS
 
 /**
@@ -302,7 +377,7 @@ Packet **UTHBuildPacketArrayFromEth(uint8_t *raw_eth[], int *pktsize, int numpkt
             SCFree(p);
             return NULL;
         }
-        DecodeEthernet(&th_v, &dtv, p[i], raw_eth[i], pktsize[i], NULL);
+        DecodeEthernet(&th_v, &dtv, p[i], raw_eth[i], pktsize[i]);
     }
     return p;
 }
@@ -326,7 +401,7 @@ Packet *UTHBuildPacketFromEth(uint8_t *raw_eth, uint16_t pktsize)
     memset(&dtv, 0, sizeof(DecodeThreadVars));
     memset(&th_v, 0, sizeof(th_v));
 
-    DecodeEthernet(&th_v, &dtv, p, raw_eth, pktsize, NULL);
+    DecodeEthernet(&th_v, &dtv, p, raw_eth, pktsize);
     return p;
 }
 
@@ -445,59 +520,13 @@ void UTHAssignFlow(Packet *p, Flow *f)
 
 Flow *UTHBuildFlow(int family, const char *src, const char *dst, Port sp, Port dp)
 {
-    struct in_addr in;
-
-    Flow *f = SCMalloc(sizeof(Flow));
-    if (unlikely(f == NULL)) {
-        printf("FlowAlloc failed\n");
-        ;
-        return NULL;
-    }
-    memset(f, 0x00, sizeof(Flow));
-
-    FLOW_INITIALIZE(f);
-
-    if (family == AF_INET) {
-        f->flags |= FLOW_IPV4;
-    } else if (family == AF_INET6) {
-        f->flags |= FLOW_IPV6;
-    }
-
-    if (src != NULL) {
-        if (family == AF_INET) {
-            if (inet_pton(AF_INET, src, &in) != 1) {
-                printf("invalid address %s\n", src);
-                SCFree(f);
-                return NULL;
-            }
-            f->src.addr_data32[0] = in.s_addr;
-        } else {
-            BUG_ON(1);
-        }
-    }
-    if (dst != NULL) {
-        if (family == AF_INET) {
-            if (inet_pton(AF_INET, dst, &in) != 1) {
-                printf("invalid address %s\n", dst);
-                SCFree(f);
-                return NULL;
-            }
-            f->dst.addr_data32[0] = in.s_addr;
-        } else {
-            BUG_ON(1);
-        }
-    }
-
-    f->sp = sp;
-    f->dp = dp;
-
-    return f;
+    return TestHelperBuildFlow(family, src, dst, sp, dp);
 }
 
 void UTHFreeFlow(Flow *flow)
 {
     if (flow != NULL) {
-        FlowFree(flow);
+        SCFree(flow);//FlowFree(flow);
     }
 }
 
@@ -901,6 +930,9 @@ end:
 
 uint32_t UTHBuildPacketOfFlows(uint32_t start, uint32_t end, uint8_t dir)
 {
+    FlowLookupStruct fls;
+    memset(&fls, 0, sizeof(fls));
+
     uint32_t i = start;
     uint8_t payload[] = "Payload";
     for (; i < end; i++) {
@@ -912,9 +944,9 @@ uint32_t UTHBuildPacketOfFlows(uint32_t start, uint32_t end, uint8_t dir)
             p->src.addr_data32[0] = i + 1;
             p->dst.addr_data32[0] = i;
         }
-        FlowHandlePacket(NULL, NULL, p);
+        FlowHandlePacket(NULL, &fls, p);
         if (p->flow != NULL) {
-            SC_ATOMIC_RESET(p->flow->use_cnt);
+            p->flow->use_cnt = 0;
             FLOWLOCK_UNLOCK(p->flow);
         }
 
@@ -922,6 +954,14 @@ uint32_t UTHBuildPacketOfFlows(uint32_t start, uint32_t end, uint8_t dir)
         UTHFreePacket(p);
     }
 
+    Flow *f;
+    while ((f = FlowQueuePrivateGetFromTop(&fls.spare_queue))) {
+        FlowFree(f);
+    }
+    while ((f = FlowQueuePrivateGetFromTop(&fls.work_queue))) {
+        FlowFree(f);
+    }
+
     return i;
 }
 
@@ -942,7 +982,6 @@ int UTHParseSignature(const char *str, bool expect)
     PASS;
 }
 
-
 /*
  * unittests for the unittest helpers
  */
@@ -995,6 +1034,18 @@ static int CheckUTHTestPacket(Packet *p, uint8_t ipproto)
     return 1;
 }
 
+#ifdef HAVE_MEMMEM
+#include <string.h>
+void * UTHmemsearch(const void *big, size_t big_len, const void *little, size_t little_len) {
+    return memmem(big, big_len, little, little_len);
+}
+#else
+#include "util-spm-bs.h"
+void * UTHmemsearch(const void *big, size_t big_len, const void *little, size_t little_len) {
+    return BasicSearch(big, big_len, little, little_len);
+}
+#endif //HAVE_MEMMEM
+
 /**
  * \brief UTHBuildPacketRealTest01 wrapper to check packets for unittests
  */
@@ -1064,11 +1115,11 @@ static int UTHBuildPacketOfFlowsTest01(void)
     int result = 0;
 
     FlowInitConfig(FLOW_QUIET);
-    uint32_t flow_spare_q_len = flow_spare_q.len;
+    uint32_t flow_spare_q_len = FlowSpareGetPoolSize();
 
     UTHBuildPacketOfFlows(0, 100, 0);
 
-    if (flow_spare_q.len != flow_spare_q_len - 100)
+    if (FlowSpareGetPoolSize() != flow_spare_q_len - 100)
         result = 0;
     else
         result = 1;
